@@ -79,6 +79,7 @@ public:
     void TxDelay(void);
     void ConnectPeriodic(void);
     bool my_connect(bool close_on_fail);
+    void mySockInit(void);
     bool isConnected(void) const;
     int SendFirstPayload(void);
     int BarrierClient(struct BarrierMutex *);
@@ -97,6 +98,7 @@ private:
     void AwaitServerFinPacket(void);
     bool InProgress(void);
     void PostNullEvent(void);
+    void PostNullEvent(bool isFirst);
     void AwaitServerCloseEvent(void);
     inline void tcp_shutdown(void);
     bool connected;
@@ -108,7 +110,6 @@ private:
     bool apply_first_udppkt_delay;
     int udp_payload_minimum;
     void myReportPacket(void);
-
     // TCP plain
     void RunTCP(void);
     // TCP version which supports rate limiting per -b
@@ -138,6 +139,7 @@ private:
     Timestamp connect_done, connect_start;
     Isochronous::FrameCounter *framecounter;
     bool isburst;
+    bool mysock_init_done;
     bool peerclose;
     Timestamp write_start;
 }; // end class Client
