@@ -205,6 +205,10 @@ extern const char report_sumcnt_bw_read_enhanced_header[];
 
 extern const char report_sumcnt_bw_read_enhanced_format[];
 
+extern const char report_sumcnt_bw_read_triptime_header[];
+
+extern const char report_sumcnt_bw_read_triptime_format[];
+
 extern const char report_sumcnt_bw_pps_enhanced_header[];
 
 extern const char report_sumcnt_bw_pps_enhanced_format[];
@@ -316,6 +320,15 @@ extern const char report_client_bb_bw_triptime_format[];
 extern const char report_bw_isoch_enhanced_netpwr_header[];
 
 extern const char report_bw_isoch_enhanced_netpwr_format[];
+
+extern const char report_sumcnt_udp_enhanced_header[];
+
+extern const char report_sumcnt_udp_enhanced_format[];
+
+extern const char report_sumcnt_udp_triptime_header[];
+
+extern const char report_sumcnt_udp_triptime_format[];
+
 /* -------------------------------------------------------------------
  * Misc reports
  * ------------------------------------------------------------------- */
@@ -352,8 +365,14 @@ extern const char reportCSV_peer[];
 
 extern const char reportCSV_bw_format[];
 
+extern const char reportCSV_bw_read_enhanced_format[];
+
+extern const char reportCSV_bw_write_enhanced_format[];
+
 extern const char reportCSV_bw_jitter_loss_format[];
 
+extern const char reportCSV_bw_jitter_loss_pps_format[];
+
 /* -------------------------------------------------------------------
  * warnings
  * ------------------------------------------------------------------- */
