@@ -76,6 +76,11 @@ struct server_hdr;
 // latency output. Units are seconds
 #define UNREALISTIC_LATENCYMINMIN -0.01*1e6
 #define UNREALISTIC_LATENCYMINMAX 60*1e6
+#define JITTER_BINCNT 1000000 // 1 million
+#define JITTER_BINWIDTH 100 // 100 usecs
+#define JITTER_UNITS 1e6 // usecs
+#define JITTER_LCI 5 // 5%
+#define JITTER_UCI 95 // 95%
 
 #ifdef __cplusplus
 extern "C" {
@@ -83,7 +88,6 @@ extern "C" {
 
 extern struct Condition ReportCond;
 extern struct Condition ReportsPending;
-extern int groupID;
 extern Mutex transferid_mutex;
 
 /*
@@ -100,24 +104,25 @@ struct MeanMinMaxStats {
     double mean;
     double m2;
     double vd;
-    int cnt;
-    int err;
+    intmax_t cnt;
+    intmax_t err;
 };
 
 #define TCPREADBINCOUNT 8
 struct ReadStats {
-    int cntRead;
-    int totcntRead;
+    intmax_t cntRead;
+    intmax_t totcntRead;
     int bins[TCPREADBINCOUNT];
     int totbins[TCPREADBINCOUNT];
     int binsize;
 };
 
 struct WriteStats {
-    int WriteCnt;
-    int WriteErr;
-    int totWriteCnt;
-    int totWriteErr;
+    intmax_t WriteCnt;
+    intmax_t WriteErr;
+    intmax_t totWriteCnt;
+    intmax_t totWriteErr;
+    intmax_t totWriteWarn;
     struct iperf_tcpstats tcpstats;
 };
 
@@ -133,7 +138,7 @@ enum WriteErrType {
     WriteNoErr  = 0,
     WriteErrAccount,
     WriteErrFatal,
-    WriteErrNoAccount,
+    WriteErrNoAccount
 };
 
 struct L2Stats {
@@ -230,6 +235,8 @@ struct ReportCommon {
     double FPS;
     int bbsize;
     int bbhold;
+    int bbcount;
+    int jitter_binwidth;
 #if WIN32
     SOCKET socket;
 #else
@@ -283,6 +290,8 @@ struct IsochStats {
     unsigned int mBurstInterval;
     unsigned int mBurstIPG; //IPG of packets within the burst
     uint32_t frameID;
+    struct RunningMMMStats transit;
+    int newburst;
 };
 
 struct ReportSettings {
@@ -339,7 +348,9 @@ struct TransferInfo {
     union SendReadStats sock_callstats;
     struct IsochStats isochstats;
     struct histogram *latency_histogram;
+    struct histogram *jitter_histogram;
     struct RunningMMMStats transit;
+    struct RunningMMMStats inline_jitter; // per RTP inline calc
     struct histogram *framelatency_histogram;
     struct RunningMMMStats frame; // isochronous frame or msg burst
     struct histogram *bbrtt_histogram;
@@ -347,6 +358,7 @@ struct TransferInfo {
     struct RunningMMMStats bbowdto;
     struct RunningMMMStats bbowdfro;
     struct RunningMMMStats bbasym;
+    struct MeanMinMaxStats schedule_error;
     struct L2Stats l2counts;
     // Packet and frame state info
     uint32_t matchframeID;
@@ -357,15 +369,23 @@ struct TransferInfo {
     bool isEnableTcpInfo;
     struct RunningMMMStats write_mmm;
     struct histogram *write_histogram;
+    double iBBrunning;
+    double fBBrunning;
+    double iInP;
+    double fInP;
+    double iInPVar;
+    double fInPVar;
 };
 
 struct SumReport {
     struct ReferenceMutex reference;
     int threads;
+    int threads_cntr_fsum;
     struct TransferInfo info;
     void (*transfer_protocol_sum_handler) (struct TransferInfo *stats, int final);
     struct BarrierMutex fullduplex_barrier;
     int sum_fd_set;
+    bool sum_reverse_set;
 };
 
 struct ReporterData {
@@ -433,38 +453,44 @@ extern report_serverstatistics serverstatistics_reports[];
 extern report_statistics multiple_reports[];
 
 
-// Packet accounting routines
-void reporter_handle_packet_null(struct ReporterData *report, struct ReportStruct *packet);
+// The packet or read/write accounting routines used by the reporter thread
+// They're function vectors called from the reporter loop via
+// this_ireport->packet_handler_pre_report or this_ireport->packet_handler_post_report
+// The vectors are set during report instantiation in Reports.c
+// Note that client packet handling is the same for TCP and UDP
+void reporter_handle_packet_null(struct ReporterData *data, struct ReportStruct *packet);
+void reporter_handle_packet_client(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_server_udp(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
-void reporter_handle_packet_client(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_bb_client(struct ReporterData *data, struct ReportStruct *packet);
 void reporter_handle_packet_bb_server(struct ReporterData *data, struct ReportStruct *packet);
 
-// Reporter's conditional prints, right now have time and frame based sampling, possibly add packet based
+// Reporter thread's conditional prints of interval reports
+// Invoked from the Reporter thread per function vector this_ireport->transfer_interval_handler
+// This is set during Report instantiation (found in Reports.c)
+// These conditionally, e.g per a sample interval, invoke the transfer protocol handlers
 int reporter_condprint_time_interval_report(struct ReporterData *data, struct ReportStruct *packet);
-int reporter_condprint_frame_interval_report_client_udp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_client_udp(struct ReporterData *data, struct ReportStruct *packet);
 int reporter_condprint_frame_interval_report_server_udp(struct ReporterData *data, struct ReportStruct *packet);
 int reporter_condprint_frame_interval_report_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
-int reporter_condprint_frame_interval_report_client_tcp(struct ReporterData *reporthdr, struct ReportStruct *packet);
-int reporter_condprint_burst_interval_report_client_udp(struct ReporterData *reporthdr, struct ReportStruct *packet);
+int reporter_condprint_frame_interval_report_client_tcp(struct ReporterData *data, struct ReportStruct *packet);
+int reporter_condprint_burst_interval_report_client_udp(struct ReporterData *data, struct ReportStruct *packet);
 int reporter_condprint_burst_interval_report_server_udp(struct ReporterData *data, struct ReportStruct *packet);
 int reporter_condprint_burst_interval_report_server_tcp(struct ReporterData *data, struct ReportStruct *packet);
-int reporter_condprint_burst_interval_report_client_tcp(struct ReporterData *reporthdr, struct ReportStruct *packet);
-//void reporter_set_timestamps_time(struct ReporterData *stats, enum TimestampType);
+int reporter_condprint_burst_interval_report_client_tcp(struct ReporterData *data, struct ReportStruct *packet);
 
-// Reporter's interval output specialize routines
-void reporter_transfer_protocol_null(struct ReporterData *stats, int final);
-//void reporter_transfer_protocol_reports(struct ReporterData *stats, struct ReportStruct *packet);
-//void reporter_transfer_protocol_multireports(struct ReporterData *stats, struct ReportStruct *packet);
+// Each report type needs specialized updating prior to printing its report
+// These functions realize that updating. They are called by the transfer protocol handler,
+// e.g. (*data->transfer_protocol_handler)(data, 0) found in src/Reporter.c
+// These update the TransferInfo stats struct which is used during output (e.g. the output handler)
+void reporter_transfer_protocol_null(struct ReporterData *data, int final);
 void reporter_transfer_protocol_client_tcp(struct ReporterData *data, int final);
 void reporter_transfer_protocol_client_bb_tcp(struct ReporterData *data, int final);
 void reporter_transfer_protocol_client_udp(struct ReporterData *data, int final);
 void reporter_transfer_protocol_server_tcp(struct ReporterData *data, int final);
 void reporter_transfer_protocol_server_bb_tcp(struct ReporterData *data, int final);
 void reporter_transfer_protocol_server_udp(struct ReporterData *data, int final);
-
-// Reporter's sum output routines (per -P > 1)
+// Function vectors to suppport sum reports
 void reporter_transfer_protocol_sum_client_tcp(struct TransferInfo *stats, int final);
 void reporter_transfer_protocol_sum_server_tcp(struct TransferInfo *stats, int final);
 void reporter_transfer_protocol_sum_client_udp(struct TransferInfo *stats, int final);
@@ -472,20 +498,22 @@ void reporter_transfer_protocol_sum_server_udp(struct TransferInfo *stats, int f
 void reporter_transfer_protocol_fullduplex_tcp(struct TransferInfo *stats, int final);
 void reporter_transfer_protocol_fullduplex_udp(struct TransferInfo *stats, int final);
 
-
-// Reporter print routines
+// Report output print routines invoked by the transfer_protocol handler
+// Bound in Report instantiation and invoked by the transfer protocol
+// handler, using (*stats->output_handler)(stats)
 // TCP server
 void tcp_output_read(struct TransferInfo *stats);
 void tcp_output_read_enhanced(struct TransferInfo *stats);
-void tcp_output_read_enhanced_triptime(struct TransferInfo *stats);
+void tcp_output_read_triptime(struct TransferInfo *stats);
 void tcp_output_sum_read(struct TransferInfo *stats);
 void tcp_output_sum_read_enhanced(struct TransferInfo *stats);
 void tcp_output_sumcnt_read(struct TransferInfo *stats);
 void tcp_output_sumcnt_read_enhanced (struct TransferInfo *stats);
+void tcp_output_sumcnt_read_triptime (struct TransferInfo *stats);
 void tcp_output_frame_read(struct TransferInfo *stats);
 void tcp_output_frame_read_triptime(struct TransferInfo *stats);
 void tcp_output_burst_read(struct TransferInfo *stats);
-
+void tcp_output_read_enhanced_isoch(struct TransferInfo *stats);
 // TCP client
 void tcp_output_write(struct TransferInfo *stats);
 void tcp_output_burst_write(struct TransferInfo *stats);
@@ -501,16 +529,15 @@ void tcp_output_write_bb(struct TransferInfo *stats);
 void tcp_output_fullduplex(struct TransferInfo *stats);
 void tcp_output_fullduplex_enhanced(struct TransferInfo *stats);
 void tcp_output_fullduplex_sum (struct TransferInfo *stats);
-
 // UDP server
 void udp_output_read(struct TransferInfo *stats);
-void udp_output_read_enhanced_triptime(struct TransferInfo *stats);
-void udp_output_read_enhanced_triptime_isoch(struct TransferInfo *stats);
+void udp_output_read_enhanced(struct TransferInfo *stats);
+void udp_output_read_triptime(struct TransferInfo *stats);
+void udp_output_read_triptime_isoch(struct TransferInfo *stats);
 void udp_output_sum_read(struct TransferInfo *stats);
 void udp_output_sum_read_enhanced (struct TransferInfo *stats);
 void udp_output_sumcnt(struct TransferInfo *stats);
 void udp_output_sumcnt_read_enhanced (struct TransferInfo *stats);
-
 //UDP client
 void udp_output_write(struct TransferInfo *stats);
 void udp_output_sum_write(struct TransferInfo *stats);
@@ -520,17 +547,20 @@ void udp_output_sum_write_enhanced (struct TransferInfo *stats);
 void udp_output_sumcnt_write(struct TransferInfo *stats);
 void udp_output_sumcnt_write_enhanced (struct TransferInfo *stats);
 void udp_output_sumcnt_enhanced(struct TransferInfo *stats);
+void udp_output_sumcnt_read_triptime(struct TransferInfo *stats);
 // UDP full duplex
 void udp_output_fullduplex(struct TransferInfo *stats);
 void udp_output_fullduplex_enhanced(struct TransferInfo *stats);
 void udp_output_fullduplex_sum(struct TransferInfo *stats);
 
-
 // CSV output
 void udp_output_basic_csv(struct TransferInfo *stats);
+void udp_output_enhanced_csv(struct TransferInfo *stats);
 void tcp_output_basic_csv(struct TransferInfo *stats);
+void tcp_output_read_enhanced_csv(struct TransferInfo *stats);
+void tcp_output_write_enhanced_csv(struct TransferInfo *stats);
 
-// Rest of the reporter output routines
+// The report output routines that are simpler and aren't related to stats
 void reporter_print_connection_report(struct ConnectionInfo *report);
 void reporter_print_settings_report(struct ReportSettings *report);
 void reporter_print_server_relay_report(struct ServerRelay *report);
@@ -545,6 +575,7 @@ int reporter_process_transfer_report (struct ReporterData *this_ireport);
 int reporter_process_report (struct ReportHeader *reporthdr);
 
 void setTransferID(struct thread_Settings *inSettings, int role_reversal);
+void format_ips_port_string (struct TransferInfo *stats, bool sum);
 
 #ifdef __cplusplus
 } /* end extern "C" */
