@@ -72,12 +72,13 @@ extern "C" {
 #define HEADER_VERSION1      0x80000000
 #define HEADER_EXTEND        0x40000000
 #define HEADER_UDPTESTS      0x20000000
-#define HEADER_BOUNCEBACK    0x10000000
+// reserved for keycheck flag    0x10000000
 #define HEADER_SEQNO64B      0x08000000
 #define HEADER_VERSION2      0x04000000
 #define HEADER_V2PEERDETECT  0x02000000
 #define HEADER_UDPAVOID2     0x02000000
 #define HEADER_UDPAVOID1     0x01000000
+#define HEADER_BOUNCEBACK    0x00800000
 
 #define HEADER32_SMALL_TRIPTIMES 0x00020000
 #define HEADER_LEN_BIT       0x00010000
@@ -91,6 +92,7 @@ extern "C" {
 #define HEADER_BBQUICKACK    0x8000
 #define HEADER_BBCLOCKSYNCED 0x4000 // used in the bb header only
 #define HEADER_BBTOS         0x2000
+#define HEADER_BBSTOP        0x1000
 
 // newer flags available per HEADER_EXTEND
 // Below flags are used to pass test settings in *every* UDP packet
@@ -221,6 +223,7 @@ struct bounceback_hdr {
     uint32_t bbid;
     uint16_t bbflags;
     uint16_t tos;
+    uint32_t bbRunTime; //units 10 ms
     struct bb_ts bbclientTx_ts;
     struct bb_ts bbserverRx_ts;
     struct bb_ts bbserverTx_ts;
@@ -273,7 +276,7 @@ struct client_hdrext_starttime_fq {
  *                +--------+--------+--------+--------+
  *            3   |        flags                      |
  *                +--------+--------+--------+--------+
- *            4   |        isoch burst period (s)     |
+ *            4   |        reserved                   |
  *                +--------+--------+--------+--------+
  *            5   |        isoch burst period (us)    |
  *                +--------+--------+--------+--------+
@@ -330,7 +333,7 @@ struct TCP_burst_payload {
     uint32_t start_tv_sec;
     uint32_t start_tv_usec;
     struct TCP_oneway_triptime send_tt;
-    uint32_t burst_period_s;
+    uint32_t reserved0;
     uint32_t burst_period_us;
     uint32_t burst_id;
     uint32_t burst_size;
