@@ -168,8 +168,6 @@ void server_spawn(struct thread_Settings *thread) {
 
 static void clientside_client_basic (struct thread_Settings *thread, Client *theClient) {
     setTransferID(thread, 0);
-    SockAddr_remoteAddr(thread);
-    theClient->my_connect(true);
 #ifdef HAVE_THREAD_DEBUG
     if (isBounceBack(thread)) {
 	thread_debug("Launch: spawn client bounce-back mode, size = %d", thread->mBurstSize);
@@ -177,6 +175,19 @@ static void clientside_client_basic (struct thread_Settings *thread, Client *the
 	thread_debug("Launch: client spawn thread basic (sock=%d)", thread->mSock);
     }
 #endif
+    SockAddr_remoteAddr(thread);
+    // Bounceback start delays the connect too
+    if (isBounceBack(thread)) {
+	if (isTxStartTime(thread)) {
+	    theClient->mySockInit();
+	    clock_usleep_abstime(&thread->txstart_epoch);
+	} else if (isTxHoldback(thread)) {
+	    unsetTxHoldback(thread);
+	    theClient->mySockInit();
+	    clock_usleep(&thread->txholdback_timer);
+	}
+    }
+    theClient->my_connect(true);
     if ((thread->mThreads > 1) && !isNoConnectSync(thread) && !isCompat(thread))
 	// When -P > 1 then all threads finish connect before starting traffic
 	theClient->BarrierClient(thread->connects_done);
@@ -188,7 +199,8 @@ static void clientside_client_basic (struct thread_Settings *thread, Client *the
     }
 }
 
-static void clientside_client_reverse (struct thread_Settings *thread, Client *theClient) {
+static void clientside_client_reverse (struct thread_Settings *thread,  \
+				       struct thread_Settings *reverse_client, Client *theClient) {
     setTransferID(thread, 0);
     SockAddr_remoteAddr(thread);
     theClient->my_connect(true);
@@ -199,8 +211,6 @@ static void clientside_client_reverse (struct thread_Settings *thread, Client *t
 	// When -P > 1 then all threads finish connect before starting traffic
 	theClient->BarrierClient(thread->connects_done);
     if (theClient->isConnected()) {
-	struct thread_Settings *reverse_client = NULL;
-	Settings_Copy(thread, &reverse_client, 0);
 	FAIL((!reverse_client || !(thread->mSock > 0)), "Reverse test failed to start per thread settings or socket problem",  thread);
 	setTransferID(reverse_client, 1);
 	theClient->StartSynch();
@@ -208,6 +218,8 @@ static void clientside_client_reverse (struct thread_Settings *thread, Client *t
 	reverse_client->mThreadMode = kMode_Server;
 	setReverse(reverse_client);
 	setNoUDPfin(reverse_client); // disable the fin report - no need
+	reverse_client->size_local = sizeof(iperf_sockaddr);
+	getsockname(reverse_client->mSock, reinterpret_cast<sockaddr*>(&reverse_client->local), &reverse_client->size_local);
         if ((thread->mThreads > 1) || isSumOnly(thread))
 	    Iperf_push_host(reverse_client);
 	thread_start(reverse_client);
@@ -216,14 +228,14 @@ static void clientside_client_reverse (struct thread_Settings *thread, Client *t
     }
 }
 
-static void clientside_client_fullduplex (struct thread_Settings *thread, Client *theClient) {
-    struct thread_Settings *reverse_client = NULL;
+static void clientside_client_fullduplex (struct thread_Settings *thread, \
+					  struct thread_Settings *reverse_client, Client *theClient) {
     setTransferID(thread, 0);
     SockAddr_remoteAddr(thread);
     if (!isBounceBack(thread)) {
         thread->mFullDuplexReport = InitSumReport(thread, -1, 1);
     }
-    Settings_Copy(thread, &reverse_client, 0);
+    Settings_Copy(thread, &reverse_client, SHALLOW_COPY);
     if ((thread->mThreads > 1) || isSumOnly(thread) || \
 	(!(thread->mThreads > 1) && !isEnhanced(thread))) {
 	Iperf_push_host(thread);
@@ -241,7 +253,6 @@ static void clientside_client_fullduplex (struct thread_Settings *thread, Client
     if (theClient->isConnected()) {
 	thread->mFullDuplexReport->info.common->socket = thread->mSock;
 	FAIL((!reverse_client || !(thread->mSock > 0)), "Reverse test failed to start per thread settings or socket problem",  thread);
-	reverse_client->mSumReport = thread->mSumReport;
 	reverse_client->mSock = thread->mSock; // use the same socket for both directions
 	reverse_client->mThreadMode = kMode_Server;
 	setReverse(reverse_client);
@@ -272,7 +283,13 @@ static void serverside_client_bidir (struct thread_Settings *thread, Client *the
     setTransferID(thread, 1);
     SockAddr_zeroAddress(&thread->peer);
     SockAddr_remoteAddr(thread);
-    unsetNoSettReport(thread);
+    if (thread->mReportMode == kReport_CSV) {
+	setNoSettReport(thread);
+	setNoConnReport(thread);
+    } else {
+	unsetNoSettReport(thread);
+	unsetNoConnReport(thread);
+    }
     setReport(thread);
     theClient->my_connect(false);
     if (theClient->isConnected()) {
@@ -323,12 +340,18 @@ void client_spawn (struct thread_Settings *thread) {
 	if (!isReverse(thread) && !isFullDuplex(thread)) {
 	    clientside_client_basic(thread, theClient);
 	} else if (isReverse(thread) && !isFullDuplex(thread)) {
-	    clientside_client_reverse(thread, theClient);
+	    struct thread_Settings *reverse_thread = NULL;
+	    Settings_Copy(thread, &reverse_thread, DEEP_COPY);
+	    FAIL((reverse_thread == NULL), "Reverse thread alloc failed",  thread);
+	    clientside_client_reverse(thread, reverse_thread, theClient);
 	} else if (isFullDuplex(thread)) {
-	    clientside_client_fullduplex(thread, theClient);
+	    struct thread_Settings *reverse_thread = NULL;
+	    Settings_Copy(thread, &reverse_thread, DEEP_COPY);
+	    FAIL((reverse_thread == NULL), "Reverse in full-duplex thread alloc failed",  thread);
+	    clientside_client_fullduplex(thread, reverse_thread, theClient);
 	} else {
 	    fprintf(stdout, "Program error in client side client_spawn");
-	    _exit(0);
+	    _exit(-1);
 	}
     } else {
 	if (thread->mMode != kTest_Normal) {
@@ -371,7 +394,7 @@ void client_init(struct thread_Settings *clients) {
     // provided settings, unsetting the report flag and add
     // to the list of threads to start
     for (int i = 1; i < clients->mThreads; i++) {
-	Settings_Copy(clients, &next, 1);
+	Settings_Copy(clients, &next, DEEP_COPY);
 	// printf("*****port/thread = %d/%d\n", next->mPort + i, i);
 	if (next) {
 	    if (isIncrSrcIP(clients) && (clients->mLocalhost != NULL)) {
@@ -396,21 +419,27 @@ void client_init(struct thread_Settings *clients) {
 	itr->runNow = next;
 	itr = next;
     }
-    if (isBounceBack(clients) && (isWorkingLoadUp(clients) || isWorkingLoadDown(clients))) {
-	int bbwlthreads = (clients->mBounceBackCongestThreads == 0) ? 1 : clients->mBounceBackCongestThreads;
-	while (bbwlthreads--) {
-	    Settings_Copy(clients, &next, 1);
+    if ((isBounceBack(clients) || isConnectOnly(clients) || isPeriodicBurst(clients)) \
+	&& (isWorkingLoadUp(clients) || isWorkingLoadDown(clients))) {
+	int working_load_threads = (clients->mWorkingLoadThreads == 0) ? 1 : clients->mWorkingLoadThreads;
+	while (working_load_threads--) {
+	    Settings_Copy(clients, &next, DEEP_COPY);
 	    if (next != NULL) {
 		unsetBounceBack(next);
+		unsetConnectOnly(next);
 		unsetPeriodicBurst(next);
 		unsetTxHoldback(next); // don't delay working load threads
 		next->mTOS = 0; // disable any QoS on the congestion stream
+#if HAVE_DECL_TCP_NOTSENT_LOWAT
+		next->mWritePrefetch = SMALL_WRITE_PREFETCH;
+		setWritePrefetch(next);
+#endif
 		if (isWorkingLoadUp(clients) && isWorkingLoadDown(clients)) {
 		    setFullDuplex(next);
 		} else if (isWorkingLoadDown(clients)) {
 		    setReverse(next);
 		}
-		if (isBounceBack(clients) && (clients->mBounceBackCongestThreads > 1)) {
+		if (isBounceBack(clients) && (clients->mWorkingLoadThreads > 1)) {
 		    Iperf_push_host(clients);
 		}
 		// Bump the bounceback time to include the delay time
@@ -437,7 +466,7 @@ void listeners_init(struct thread_Settings *listener) {
     struct thread_Settings *itr = listener;
     struct thread_Settings *next = NULL;
     for (int ix = 1; ix < (listener->mPortLast - listener->mPort + 1); ix++)  {
-	Settings_Copy(listener, &next, 1);
+	Settings_Copy(listener, &next, DEEP_COPY);
 	if (next != NULL) {
 	    setNoSettReport(next);
 	    next->mPort = listener->mPort + ix;
