@@ -184,7 +184,8 @@ void Listener::Run () {
 	}
 	// Use a select() with a timeout if -t is set or if this is a v1 -r or -d test
 	fd_set set;
-	if ((mMode_Time) || isCompat(mSettings) || isPermitKey(mSettings)) {
+	if (((mMode_Time || isCompat(mSettings)) && (!isPermitKey(mSettings) && (mSettings->mAmount != 0))) \
+	    || (isPermitKey(mSettings) && (mSettings->mListenerTimeout != 0))) {
 	    // Hang a select w/timeout on the listener socket
 	    struct timeval timeout;
 	    if (!isPermitKey(mSettings)) {
@@ -217,7 +218,7 @@ void Listener::Run () {
 	    WARN(1, "Failed setting socket to blocking mode");
 	}
 	// Instantiate another settings object to be used by the server thread
-	Settings_Copy(mSettings, &server, 1);
+	Settings_Copy(mSettings, &server, DEEP_COPY);
 	FAIL(!server, "Failed memory allocation for server settings", mSettings);
 	server->mThreadMode = kMode_Server;
 	if (!isDataReport(mSettings))
@@ -260,9 +261,6 @@ void Listener::Run () {
 	    // Not allowed, reset things and restart the loop
 	    // Don't forget to delete the UDP entry (inserted in my_accept)
 	    Iperf_remove_host(server);
-	    if (DecrSumReportRefCounter(server->mSumReport) <= 0) {
-		FreeSumReport(server->mSumReport);
-	    }
 	    if (!isUDP(server))
 	        close(server->mSock);
 	    assert(server != mSettings);
@@ -291,9 +289,6 @@ void Listener::Run () {
 		PostReport(reporthdr);
 	    }
 	    Iperf_remove_host(server);
-	    if (DecrSumReportRefCounter(server->mSumReport) <= 0) {
-		FreeSumReport(server->mSumReport);
-	    }
 	    close(server->mSock);
 	    assert(server != mSettings);
 	    Settings_Destroy(server);
@@ -305,9 +300,6 @@ void Listener::Run () {
 	    if (!isCompat(mSettings) && !isTapDev(mSettings) && (isL2LengthCheck(mSettings) || isL2LengthCheck(server)) && !L2_setup(server, server->mSock)) {
 		// Requested L2 testing but L2 setup failed
 		Iperf_remove_host(server);
-		if (DecrSumReportRefCounter(server->mSumReport) <= 0) {
-		    FreeSumReport(server->mSumReport);
-		}
 		assert(server != mSettings);
 		Settings_Destroy(server);
 		continue;
@@ -318,6 +310,12 @@ void Listener::Run () {
 	    setSeqNo64b(server);
 	}
 	setTransferID(server, 0);
+	if ((mSettings->mReportMode == kReport_CSV) && server->mSumReport && !server->mSumReport->sum_reverse_set) {
+	    format_ips_port_string(&server->mSumReport->info, 1);
+	    server->mSumReport->sum_reverse_set = true;
+	} else if (isTripTime(server) && server->mSumReport) {
+	    SetSumHandlers(server, server->mSumReport); // this needs to be done again after first read
+	}
 
 	// Read any more test settings and test values (not just the flags) and instantiate
 	// any settings objects for client threads (e.g. bidir or full duplex)
@@ -468,7 +466,7 @@ void Listener::my_listen () {
     // listen for connections (TCP only).
     // use large (INT_MAX) backlog allowing multiple simultaneous connections
     if (!isUDP(mSettings)) {
-	if (isSingleClient(mSettings) || isPermitKey(mSettings)) {
+	if (isSingleClient(mSettings)) {
 	    rc = listen(ListenSocket, 0 + mSettings->mThreads);
 	} else {
 	    rc = listen(ListenSocket, INT_MAX);
@@ -603,7 +601,7 @@ void Listener::my_multicast_join () {
 #endif
 		rc = -1;
 #if HAVE_DECL_MCAST_JOIN_SOURCE_GROUP
-		rc = setsockopt(ListenSocket,IPPROTO_IPV6,MCAST_JOIN_SOURCE_GROUP, &group_source_req,
+		rc = setsockopt(ListenSocket,IPPROTO_IPV6,MCAST_JOIN_SOURCE_GROUP, reinterpret_cast<const char *>(&group_source_req),
 				sizeof(group_source_req));
 #endif
 		FAIL_errno(rc == SOCKET_ERROR, "mcast v6 join source group",mSettings);
@@ -622,7 +620,7 @@ void Listener::my_multicast_join () {
 		group->sin6_port = 0;    /* Ignored */
 		rc = -1;
 #if HAVE_DECL_MCAST_JOIN_GROUP
-		rc = setsockopt(ListenSocket,IPPROTO_IPV6,MCAST_JOIN_GROUP, &group_req,
+		rc = setsockopt(ListenSocket,IPPROTO_IPV6,MCAST_JOIN_GROUP, reinterpret_cast<const char *>(&group_req),
 				sizeof(group_source_req));
 #endif
 		FAIL_errno(rc == SOCKET_ERROR, "mcast v6 join group",mSettings);
@@ -670,7 +668,7 @@ void Listener::my_multicast_join () {
 		rc = -1;
 
 #if HAVE_DECL_MCAST_JOIN_SOURCE_GROUP
-		rc = setsockopt(ListenSocket,IPPROTO_IP,MCAST_JOIN_SOURCE_GROUP, &group_source_req,
+		rc = setsockopt(ListenSocket,IPPROTO_IP,MCAST_JOIN_SOURCE_GROUP, reinterpret_cast<const char *>(&group_source_req),
 				sizeof(group_source_req));
 #endif
 
@@ -707,7 +705,7 @@ void Listener::my_multicast_join () {
 		group->sin_port = 0;    /* Ignored */
 		rc = -1;
 #if HAVE_DECL_MCAST_JOIN_GROUP
-		rc = setsockopt(ListenSocket,IPPROTO_IP,MCAST_JOIN_GROUP, &group_req,
+		rc = setsockopt(ListenSocket,IPPROTO_IP,MCAST_JOIN_GROUP, reinterpret_cast<const char *>(&group_req),
 				sizeof(group_source_req));
 #endif
 		FAIL_errno(rc == SOCKET_ERROR, "mcast join group",mSettings);
@@ -1087,7 +1085,7 @@ inline bool Listener::test_permit_key(uint32_t flags, thread_Settings *server, i
     if (!isUDP(server)) {
 	int nread = 0;
 	nread = recvn(server->mSock, reinterpret_cast<char *>(&thiskey->value), keylen, 0);
-	FAIL_errno((nread < (keyoffset + keylen)), "read key", server);
+	FAIL_errno((nread < keylen), "read key", server);
     }
     strncpy(server->mPermitKey, thiskey->value, MAX_PERMITKEY_LEN + 1);
     if (strncmp(server->mPermitKey, mSettings->mPermitKey, keylen) != 0) {
@@ -1189,6 +1187,10 @@ bool Listener::apply_client_settings_udp (thread_Settings *server) {
 	    if (upperflags & HEADER_REVERSE)  {
 		server->mThreadMode=kMode_Client;
 		setServerReverse(server);
+		if (server->mSumReport != NULL) {
+		    server->mSumReport->info.common->ThreadMode=kMode_Client;
+		    setServerReverse(server->mSumReport->info.common);
+		}
 		setNoUDPfin(server);
 		unsetReport(server);
 	    }
@@ -1341,6 +1343,10 @@ bool Listener::apply_client_settings_tcp (thread_Settings *server) {
 			if (upperflags & HEADER_REVERSE) {
 			    server->mThreadMode=kMode_Client;
 			    setServerReverse(server);
+			    if (server->mSumReport != NULL) {
+				server->mSumReport->info.common->ThreadMode=kMode_Client;
+				setServerReverse(server->mSumReport->info.common);
+			    }
 			}
 		    }
 #if HAVE_DECL_TCP_NOTSENT_LOWAT
