@@ -1,4 +1,3 @@
-
 /*---------------------------------------------------------------
  * Copyright (c) 1999,2000,2001,2002,2003
  * The Board of Trustees of the University of Illinois
@@ -594,15 +593,14 @@ int reporter_process_transfer_report (struct ReporterData *this_ireport) {
 		}
 	    }
 	    if (sumstats) {
+		if (!this_ireport->GroupSumReport->threads_cntr_fsum)
+		    this_ireport->GroupSumReport->threads_cntr_fsum = this_ireport->GroupSumReport->reference.maxcount;
 		if (TimeDifference(sumstats->ts.packetTime, packet->packetTime) > 0) {
 		    sumstats->ts.packetTime = packet->packetTime;
 		}
-		if (DecrSumReportRefCounter(this_ireport->GroupSumReport) == 0) {
-		    if (this_ireport->GroupSumReport->transfer_protocol_sum_handler && \
-			((this_ireport->GroupSumReport->reference.maxcount > 1) || isSumOnly(this_ireport->info.common))) {
-			(*this_ireport->GroupSumReport->transfer_protocol_sum_handler)(&this_ireport->GroupSumReport->info, 1);
-		    }
-		    FreeSumReport(this_ireport->GroupSumReport);
+		if (this_ireport->GroupSumReport->transfer_protocol_sum_handler && \
+		    (--this_ireport->GroupSumReport->threads_cntr_fsum == 0) && (this_ireport->GroupSumReport->reference.maxcount > 1)) {
+		    (*this_ireport->GroupSumReport->transfer_protocol_sum_handler)(&this_ireport->GroupSumReport->info, 1);
 		}
 	    }
 	}
@@ -699,18 +697,63 @@ static void reporter_handle_packet_oneway_transit (struct TransferInfo *stats, s
         histogram_insert(stats->latency_histogram, transit, NULL);
     }
     double deltaTransit;
-    // from RFC 1889, Real Time Protocol (RTP)
-    // J = J + ( | D(i-1,i) | - J ) /
-    // Compute jitter
     deltaTransit = transit - stats->transit.current.last;
+    stats->transit.current.last = transit; // shift transit for next time
     if (deltaTransit < 0.0) {
 	deltaTransit = -deltaTransit;
     }
-    stats->jitter += (deltaTransit - stats->jitter) / (16.0);
     // Compute end/end delay stats
     reporter_update_mmm(&stats->transit.total, transit);
     reporter_update_mmm(&stats->transit.current, transit);
-    stats->transit.current.last = transit;
+    //
+    // Compute jitter though filter the case of isoch and between isoch frames
+    // or, in other words, only calculate jitter for packets within the same isoch frame
+    //
+    // Taken from RFC 1889, Real Time Protocol (RTP)
+    // J = J + ( | D(i-1,i) | - J ) /
+    //
+    // interarrival jitter:
+    //
+    // An estimate of the statistical variance of the RTP data packet
+    // interarrival time, measured in timestamp units and expressed as
+    // an unsigned integer. The interarrival jitter J is defined to be
+    // the mean deviation (smoothed absolute value) of the difference D
+    // in packet spacing at the receiver compared to the sender for a
+    // pair of packets. As shown in the equation below, this is
+    // equivalent to the difference in the "relative transit time" for
+    // the two packets; the relative transit time is the difference
+    // between a packet's RTP timestamp and the receiver's clock at the
+    // time of arrival, measured in the same units.
+    //
+    // If Si is the RTP timestamp from packet i, and Ri is the time of
+    // arrival in RTP timestamp units for packet i, then for two packets i
+    // and j, D may be expressed as
+    //
+    //             D(i,j)=(Rj-Ri)-(Sj-Si)=(Rj-Sj)-(Ri-Si)
+    //
+    // The interarrival jitter is calculated continuously as each data
+    // packet i is received from source SSRC_n, using this difference D for
+    // that packet and the previous packet i-1 in order of arrival (not
+    // necessarily in sequence), according to the formula
+    //
+    //              J=J+(|D(i-1,i)|-J)/16
+    //
+    // Whenever a reception report is issued, the current value of J is
+    // sampled. This algorithm is the optimal first-order estimator and
+    // the gain parameter 1/16 gives a good noise reduction ratio while /
+    // maintaining a reasonable rate of convergence
+    //
+    if (isIsochronous(stats->common) && stats->isochstats.newburst) {
+        --stats->isochstats.newburst; // decr the burst counter, need for RTP estimator w/isoch
+	//	printf("**** skip value %f per frame change packet %d expected %d max = %f %d\n", deltaTransit, packet->frameID, stats->isochstats.frameID, stats->inline_jitter.total.max, stats->isochstats.newburst);
+    } else if (stats->transit.total.cnt > 1) {
+	stats->jitter += (deltaTransit - stats->jitter) / (16.0);
+	reporter_update_mmm(&stats->inline_jitter.total, stats->jitter);
+	reporter_update_mmm(&stats->inline_jitter.current, stats->jitter);
+	if (stats->jitter_histogram) {
+	    histogram_insert(stats->jitter_histogram, deltaTransit, NULL);
+	}
+    }
 }
 
 
@@ -722,6 +765,7 @@ static void reporter_handle_isoch_oneway_transit_tcp (struct TransferInfo *stats
 	// very first isochronous frame
 	if (!stats->isochstats.frameID) {
 	    stats->isochstats.framecnt.current=packet->frameID;
+	    stats->isochstats.newburst = 0; // no packet/read filtering of early samples for TCP
 	}
 	// perform client and server frame based accounting
 	if ((framedelta = (packet->frameID - stats->isochstats.frameID))) {
@@ -763,6 +807,7 @@ static void reporter_handle_isoch_oneway_transit_udp (struct TransferInfo *stats
 	stats->isochstats.framecnt.current++;
 //      stats->matchframeID = packet->frameID + 1;
 	if (framedelta == 1) {
+	    stats->isochstats.newburst = 2; // set to two per RTP's pair calculation
 	    // Triptimes use the frame start time in passed in the frame header while
 	    // it's calculated from the very first start time and frame id w/o trip timees
 	    frametransit = TimeDifference(packet->packetTime, packet->isochStartTime) \
@@ -772,7 +817,8 @@ static void reporter_handle_isoch_oneway_transit_udp (struct TransferInfo *stats
 	    if (stats->framelatency_histogram) {
 		histogram_insert(stats->framelatency_histogram, frametransit, &packet->packetTime);
 	    }
-	} else {
+	} else if (framedelta > 1) {
+	    stats->isochstats.newburst = 2; // set to two per RTP's pair calculation
 	    if (stats->common->ThreadMode == kMode_Server) {
 		stats->isochstats.framelostcnt.current += framedelta;
 	    } else {
@@ -807,7 +853,7 @@ static void reporter_handle_rxmsg_oneway_transit (struct TransferInfo *stats, st
 	stats->burstid_transition = true;
     } else if (stats->burstid_transition && packet->frameID && (packet->frameID != stats->isochstats.frameID)) {
 	stats->burstid_transition = false;
-	fprintf(stderr,"%sError: expected burst id %u but got %" PRIdMAX "\n", \
+	fprintf(stderr,"%sError: expected burst id %u but got %d\n", \
 		stats->common->transferIDStr, stats->isochstats.frameID + 1, packet->frameID);
 	stats->isochstats.frameID = packet->frameID;
     }
@@ -827,7 +873,7 @@ static inline void reporter_handle_txmsg_oneway_transit (struct TransferInfo *st
 	// printf("***Burst id = %ld, transit = %f\n", packet->frameID, stats->transit.lastTransit);
 	if (isIsochronous(stats->common)) {
 	    if (packet->frameID && (packet->frameID != (stats->isochstats.frameID + 1))) {
-		fprintf(stderr,"%sError: expected burst id %u but got %" PRIdMAX "\n", \
+		fprintf(stderr,"%sError: expected burst id %u but got %d\n", \
 			stats->common->transferIDStr, stats->isochstats.frameID + 1, packet->frameID);
 	    }
 	    stats->isochstats.frameID = packet->frameID;
@@ -837,37 +883,21 @@ static inline void reporter_handle_txmsg_oneway_transit (struct TransferInfo *st
 
 static void reporter_handle_frame_isoch_oneway_transit (struct TransferInfo *stats, struct ReportStruct *packet) {
     // printf("fid=%lu bs=%lu remain=%lu\n", packet->frameID, packet->burstsize, packet->remaining);
+    if (packet->scheduled) {
+	reporter_update_mmm(&stats->schedule_error, (double)(packet->sched_err));
+    }
     if (packet->frameID && packet->transit_ready) {
 	int framedelta=0;
 	// very first isochronous frame
 	if (!stats->isochstats.frameID) {
 	    stats->isochstats.framecnt.current=packet->frameID;
 	}
-	// perform client and server frame based accounting
+	// perform frame based accounting
 	if ((framedelta = (packet->frameID - stats->isochstats.frameID))) {
 	    stats->isochstats.framecnt.current++;
 	    if (framedelta > 1) {
-		if (stats->common->ThreadMode == kMode_Server) {
-		    int lost = framedelta - (packet->frameID - packet->prevframeID);
-		    stats->isochstats.framelostcnt.current += lost;
-		} else {
-		    stats->isochstats.framelostcnt.current += (framedelta-1);
-		    stats->isochstats.slipcnt.current++;
-		}
-	    }
-	}
-	// peform frame latency checks
-	if (stats->framelatency_histogram) {
-	    // first packet of a burst and not a duplicate
-	    if ((packet->burstsize == packet->remaining) && (stats->matchframeID!=packet->frameID)) {
-		stats->matchframeID=packet->frameID;
-	    }
-	    if ((packet->packetLen == packet->remaining) && (packet->frameID == stats->matchframeID)) {
-		// last packet of a burst (or first-last in case of a duplicate) and frame id match
-		double frametransit = TimeDifference(packet->packetTime, packet->isochStartTime) \
-		    - ((packet->burstperiod * (packet->frameID - 1)) / 1000000.0);
-		histogram_insert(stats->framelatency_histogram, frametransit, NULL);
-		stats->matchframeID = 0;  // reset the matchid so any potential duplicate is ignored
+		stats->isochstats.framelostcnt.current += (framedelta-1);
+		stats->isochstats.slipcnt.current++;
 	    }
 	}
 	stats->isochstats.frameID = packet->frameID;
@@ -903,27 +933,26 @@ void reporter_handle_packet_client (struct ReporterData *data, struct ReportStru
     if (isUDP(stats->common)) {
 	stats->PacketID = packet->packetID;
 	reporter_compute_packet_pps(stats, packet);
-    } else if (packet->transit_ready) {
-	if (isIsochronous(stats->common) && packet->frameID) {
-	    reporter_handle_frame_isoch_oneway_transit(stats, packet);
-	} else if (isPeriodicBurst(stats->common) || isTripTime(stats->common)) {
-	    reporter_handle_rxmsg_oneway_transit(stats, packet);
-	}
     }
 }
 
+#define DEBUG_BB_TIMESTAMPS 0
 void reporter_handle_packet_bb_client (struct ReporterData *data, struct ReportStruct *packet) {
+    struct TransferInfo *stats = &data->info;
+    if (packet->scheduled) {
+	reporter_update_mmm(&stats->schedule_error, (double)(packet->sched_err));
+    }
     if (!packet->emptyreport && (packet->packetLen > 0)) {
-        struct TransferInfo *stats = &data->info;
 	stats->total.Bytes.current += packet->packetLen;
 	double bbrtt = TimeDifference(packet->packetTime, packet->sentTime);
 	double bbowdto = TimeDifference(packet->sentTimeRX, packet->sentTime);
 	double bbowdfro = TimeDifference(packet->packetTime, packet->sentTimeTX);
 	double asym = bbowdfro - bbowdto;
 	stats->ts.prevpacketTime = packet->packetTime;
-#if 0
-	fprintf(stderr, "BB Debug: ctx=%lx.%lx srx=%lx.%lx stx=%lx.%lx crx=%lx.%lx\n", packet->sentTime.tv_sec, packet->sentTime.tv_usec, packet->sentTimeRX.tv_sec, packet->sentTimeRX.tv_usec, packet->sentTimeTX.tv_sec, packet->sentTimeTX.tv_usec, packet->packetTime.tv_sec, packet->pAckettime.tv_usec);
+#if DEBUG_BB_TIMESTAMPS
+	fprintf(stderr, "BB Debug: ctx=%lx.%lx srx=%lx.%lx stx=%lx.%lx crx=%lx.%lx  (hex)\n", packet->sentTime.tv_sec, packet->sentTime.tv_usec, packet->sentTimeRX.tv_sec, packet->sentTimeRX.tv_usec, packet->sentTimeTX.tv_sec, packet->sentTimeTX.tv_usec, packet->packetTime.tv_sec, packet->packetTime.tv_usec);
 	fprintf(stderr, "BB Debug: ctx=%ld.%ld srx=%ld.%ld stx=%ld.%ld crx=%ld.%ld\n", packet->sentTime.tv_sec, packet->sentTime.tv_usec, packet->sentTimeRX.tv_sec, packet->sentTimeRX.tv_usec, packet->sentTimeTX.tv_sec, packet->sentTimeTX.tv_usec, packet->packetTime.tv_sec, packet->packetTime.tv_usec);
+	fprintf(stderr, "BB RTT=%f OWDTo=%f OWDFro=%f Asym=%f\n", bbrtt, bbowdto, bbowdfro, asym);
 #endif
 	stats->iBBrunning += bbrtt;
 	stats->fBBrunning += bbrtt;
@@ -1012,11 +1041,11 @@ inline void reporter_handle_packet_server_udp (struct ReporterData *data, struct
 	}
 	reporter_compute_packet_pps(stats, packet);
 	reporter_handle_packet_oneway_transit(stats, packet);
-	if (packet->transit_ready && packet->frameID) {
+	if (packet->transit_ready) {
 	    if (isIsochronous(stats->common)) {
-		reporter_handle_isoch_oneway_transit_udp(stats, packet);
+	        reporter_handle_isoch_oneway_transit_udp(stats, packet);
 	    } else if (isPeriodicBurst(stats->common)) {
-		reporter_handle_txmsg_oneway_transit(stats, packet);
+	        reporter_handle_txmsg_oneway_transit(stats, packet);
 	    }
 	}
     }
@@ -1040,10 +1069,11 @@ static inline void reporter_handle_packet_tcpistats (struct ReporterData *data,
 /*
  * Report printing routines below
  */
-static inline void reporter_set_timestamps_time (struct ReportTimeStamps *times, enum TimeStampType tstype) {
+static inline void reporter_set_timestamps_time (struct TransferInfo *stats, enum TimeStampType tstype) {
     // There is a corner case when the first packet is also the last where the start time (which comes
     // from app level syscall) is greater than the packetTime (which come for kernel level SO_TIMESTAMP)
     // For this case set the start and end time to both zero.
+    struct ReportTimeStamps *times = &stats->ts;
     if (TimeDifference(times->packetTime, times->startTime) < 0) {
 	times->iEnd = 0;
 	times->iStart = 0;
@@ -1053,23 +1083,28 @@ static inline void reporter_set_timestamps_time (struct ReportTimeStamps *times,
 	    times->iStart = times->iEnd;
 	    times->iEnd = TimeDifference(times->nextTime, times->startTime);
 	    TimeAdd(times->nextTime, times->intervalTime);
+	    stats->final = false;
 	    break;
 	case TOTAL:
 	    times->iStart = 0;
 	    times->iEnd = TimeDifference(times->packetTime, times->startTime);
+	    stats->final = true;
 	    break;
 	case FINALPARTIAL:
 	    times->iStart = times->iEnd;
 	    times->iEnd = TimeDifference(times->packetTime, times->startTime);
+	    stats->final = false;
 	    break;
 	case INTERVALPARTIAL:
 	    if ((times->iStart = TimeDifference(times->prevpacketTime, times->startTime)) < 0)
 		times->iStart = 0.0;
 	    times->iEnd = TimeDifference(times->packetTime, times->startTime);
+	    stats->final = false;
 	    break;
 	default:
 	    times->iEnd = -1;
 	    times->iStart = -1;
+	    stats->final = false;
 	    break;
 	}
     }
@@ -1079,7 +1114,7 @@ static inline void reporter_set_timestamps_time (struct ReportTimeStamps *times,
 static inline void reporter_transfer_protocol_missed_reports (struct TransferInfo *stats, struct ReportStruct *packet) {
     while (TimeDifference(packet->packetTime, stats->ts.nextTime) > TimeDouble(stats->ts.intervalTime)) {
 //	printf("**** cmp=%f/%f next %ld.%ld packet %ld.%ld id=%ld\n", TimeDifference(packet->packetTime, stats->ts.nextTime), TimeDouble(stats->ts.intervalTime), stats->ts.nextTime.tv_sec, stats->ts.nextTime.tv_usec, packet->packetTime.tv_sec, packet->packetTime.tv_usec, packet->packetID);
-	reporter_set_timestamps_time(&stats->ts, INTERVAL);
+	reporter_set_timestamps_time(stats, INTERVAL);
 	struct TransferInfo emptystats;
 	memset(&emptystats, 0, sizeof(struct TransferInfo));
 	emptystats.ts.iStart = stats->ts.iStart;
@@ -1144,6 +1179,7 @@ static inline void reporter_reset_transfer_stats_server_tcp (struct TransferInfo
 	stats->sock_callstats.read.bins[ix] = 0;
     }
     reporter_reset_mmm(&stats->transit.current);
+    reporter_reset_mmm(&stats->isochstats.transit.current);
     stats->IPGsum = 0;
 }
 
@@ -1162,6 +1198,8 @@ static inline void reporter_reset_transfer_stats_server_udp (struct TransferInfo
     stats->l2counts.unknown = 0;
     stats->l2counts.udpcsumerr = 0;
     stats->l2counts.lengtherr = 0;
+    stats->threadcnt = 0;
+    stats->iInP = 0;
     if (stats->cntDatagrams)
 	stats->IPGsum = 0;
 }
@@ -1185,10 +1223,13 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 	stats->cntError = 0;
     stats->cntDatagrams = stats->PacketID - stats->total.Datagrams.prev;
     stats->cntIPG = stats->total.IPG.current - stats->total.IPG.prev;
+
     if (stats->latency_histogram) {
         stats->latency_histogram->final = final;
     }
-
+    if (stats->jitter_histogram) {
+        stats->jitter_histogram->final = final;
+    }
     if (isIsochronous(stats->common)) {
 	stats->isochstats.cntFrames = stats->isochstats.framecnt.current - stats->isochstats.framecnt.prev;
 	stats->isochstats.cntFramesMissed = stats->isochstats.framelostcnt.current - stats->isochstats.framelostcnt.prev;
@@ -1196,10 +1237,17 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 	if (stats->framelatency_histogram) {
 	    stats->framelatency_histogram->final = final;
 	}
-
     }
     if (stats->total.Datagrams.current == 1)
 	stats->jitter = 0;
+    if (isTripTime(stats->common) && !final) {
+	double lambda =  ((stats->IPGsum > 0.0) ? (round (stats->cntIPG / stats->IPGsum)) : 0.0);
+	double meantransit = (double) ((stats->transit.current.cnt > 0) ? (stats->transit.current.sum / stats->transit.current.cnt) : 0.0);
+	double variance = (stats->transit.current.cnt < 2) ? 0 : \
+	    (sqrt(stats->transit.current.m2 / (stats->transit.current.cnt - 1)));
+	stats->iInP = (double) lambda * meantransit;
+	stats->iInPVar = (double) lambda * variance;
+    }
     if (sumstats) {
 	sumstats->total.OutofOrder.current += stats->total.OutofOrder.current - stats->total.OutofOrder.prev;
 	// assume most of the  time out-of-order packets are not
@@ -1211,6 +1259,7 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 	if (sumstats->IPGsum < stats->IPGsum)
 	    sumstats->IPGsum = stats->IPGsum;
 	sumstats->threadcnt++;
+	sumstats->iInP += stats->iInP;
     }
     if (fullduplexstats) {
 	fullduplexstats->total.Bytes.current += stats->cntBytes;
@@ -1230,13 +1279,12 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 		stats->cntError = 0;
 	    stats->cntDatagrams = stats->PacketID - stats->total.Datagrams.prev;
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 	    }
 	}
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
-	stats->final = true;
+	reporter_set_timestamps_time(stats, TOTAL);
 	stats->IPGsum = TimeDifference(stats->ts.packetTime, stats->ts.startTime);
 	stats->cntOutofOrder = stats->total.OutofOrder.current;
 	// assume most of the  time out-of-order packets are not
@@ -1254,14 +1302,36 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 	stats->l2counts.udpcsumerr = stats->l2counts.tot_udpcsumerr;
 	stats->l2counts.lengtherr = stats->l2counts.tot_lengtherr;
 	stats->transit.current = stats->transit.total;
+	if (isTripTime(stats->common)) {
+	    double lambda =  ((stats->IPGsum > 0.0) ? (round (stats->cntIPG / stats->IPGsum)) : 0.0);
+	    double meantransit = (double) ((stats->transit.total.cnt > 0) ? (stats->transit.total.sum / stats->transit.total.cnt) : 0.0);
+	    double variance = (stats->transit.total.cnt < 2) ? 0 :	\
+		(sqrt(stats->transit.total.m2 / (stats->transit.total.cnt - 1)));
+	    stats->fInP = (double) lambda * meantransit;
+	    stats->fInPVar = (double) lambda * variance;
+	    if (sumstats) {
+		sumstats->fInP += stats->fInP;
+	    }
+	}
 	if (isIsochronous(stats->common)) {
 	    stats->isochstats.cntFrames = stats->isochstats.framecnt.current;
 	    stats->isochstats.cntFramesMissed = stats->isochstats.framelostcnt.current;
 	    stats->isochstats.cntSlips = stats->isochstats.slipcnt.current;
 	}
 	if (stats->latency_histogram) {
+	    if (sumstats && sumstats->latency_histogram) {
+	        histogram_add(sumstats->latency_histogram, stats->latency_histogram);
+		sumstats->latency_histogram->final = 1;
+	    }
 	    stats->latency_histogram->final = 1;
 	}
+	if (stats->jitter_histogram) {
+	    if (sumstats && sumstats->jitter_histogram) {
+	        histogram_add(sumstats->jitter_histogram, stats->jitter_histogram);
+		sumstats->jitter_histogram->final = 1;
+	    }
+	    stats->jitter_histogram->final = 1;
+	}
 	if (stats->framelatency_histogram) {
 	    stats->framelatency_histogram->final = 1;
 	}
@@ -1274,7 +1344,7 @@ void reporter_transfer_protocol_server_udp (struct ReporterData *data, int final
 
 void reporter_transfer_protocol_sum_server_udp (struct TransferInfo *stats, int final) {
     if (final) {
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
 	stats->cntOutofOrder = stats->total.OutofOrder.current;
 	// assume most of the  time out-of-order packets are not
 	// duplicate packets, so conditionally subtract them from the lost packets.
@@ -1301,7 +1371,6 @@ void reporter_transfer_protocol_sum_server_udp (struct TransferInfo *stats, int
     if ((stats->output_handler) && !(stats->isMaskOutput))
 	(*stats->output_handler)(stats);
     if (!final) {
-	stats->threadcnt = 0;
 	// there is no packet ID for sum server reports, set it to total cnt for calculation
 	stats->PacketID = stats->total.Datagrams.current;
 	reporter_reset_transfer_stats_server_udp(stats);
@@ -1309,7 +1378,7 @@ void reporter_transfer_protocol_sum_server_udp (struct TransferInfo *stats, int
 }
 void reporter_transfer_protocol_sum_client_udp (struct TransferInfo *stats, int final) {
     if (final) {
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
 	stats->sock_callstats.write.WriteErr = stats->sock_callstats.write.totWriteErr;
 	stats->sock_callstats.write.WriteCnt = stats->sock_callstats.write.totWriteCnt;
 	stats->cntDatagrams = stats->total.Datagrams.current;
@@ -1365,7 +1434,7 @@ void reporter_transfer_protocol_client_udp (struct ReporterData *data, int final
 	    fullduplexstats->IPGsum = stats->IPGsum;
     }
     if (final) {
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
 	stats->cntBytes = stats->total.Bytes.current;
 	stats->sock_callstats.write.WriteErr = stats->sock_callstats.write.totWriteErr;
 	stats->sock_callstats.write.WriteCnt = stats->sock_callstats.write.totWriteCnt;
@@ -1403,6 +1472,20 @@ void reporter_transfer_protocol_server_tcp (struct ReporterData *data, int final
     if (stats->framelatency_histogram) {
         stats->framelatency_histogram->final = 0;
     }
+    double thisInP;
+    if (!final) {
+	double bytecnt = (double) (stats->total.Bytes.current - stats->total.Bytes.prev);
+	double lambda = (stats->IPGsum > 0.0) ? (bytecnt / stats->IPGsum) : 0.0;
+	double meantransit = (double) ((stats->transit.current.cnt > 0) ? (stats->transit.current.sum / stats->transit.current.cnt) : 0.0);
+	thisInP  = lambda * meantransit;
+	stats->iInP = thisInP;
+    } else {
+	double bytecnt = (double) stats->cntBytes;
+	double lambda = (stats->IPGsum > 0.0) ? (bytecnt / stats->IPGsum) : 0.0;
+	double meantransit = (double) ((stats->transit.total.cnt > 0) ? (stats->transit.total.sum / stats->transit.total.cnt) : 0.0);
+	thisInP  = lambda * meantransit;
+	stats->fInP = thisInP;
+    }
     if (sumstats) {
 	sumstats->threadcnt++;
 	sumstats->total.Bytes.current += stats->cntBytes;
@@ -1412,6 +1495,11 @@ void reporter_transfer_protocol_server_tcp (struct ReporterData *data, int final
 	    sumstats->sock_callstats.read.bins[ix] += stats->sock_callstats.read.bins[ix];
 	    sumstats->sock_callstats.read.totbins[ix] += stats->sock_callstats.read.bins[ix];
         }
+	if (!final) {
+	    sumstats->iInP += thisInP;
+	} else {
+	    sumstats->fInP += thisInP;
+	}
     }
     if (fullduplexstats) {
 	fullduplexstats->total.Bytes.current += stats->cntBytes;
@@ -1425,7 +1513,7 @@ void reporter_transfer_protocol_server_tcp (struct ReporterData *data, int final
 		    stats->isochstats.cntFramesMissed = stats->isochstats.framelostcnt.current - stats->isochstats.framelostcnt.prev;
 		    stats->isochstats.cntSlips = stats->isochstats.slipcnt.current - stats->isochstats.slipcnt.prev;
 		}
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 		reporter_reset_transfer_stats_server_tcp(stats);
@@ -1434,8 +1522,7 @@ void reporter_transfer_protocol_server_tcp (struct ReporterData *data, int final
         if (stats->framelatency_histogram) {
 	    stats->framelatency_histogram->final = 1;
 	}
-	stats->final = true;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
         stats->cntBytes = stats->total.Bytes.current;
 	stats->IPGsum = stats->ts.iEnd;
         stats->sock_callstats.read.cntRead = stats->sock_callstats.read.totcntRead;
@@ -1449,6 +1536,10 @@ void reporter_transfer_protocol_server_tcp (struct ReporterData *data, int final
 	}
 	stats->transit.current = stats->transit.total;
 	if (stats->framelatency_histogram) {
+	    if (sumstats && sumstats->framelatency_histogram) {
+	        histogram_add(sumstats->framelatency_histogram, stats->framelatency_histogram);
+		sumstats->framelatency_histogram->final = 1;
+	    }
 	    stats->framelatency_histogram->final = 1;
 	}
     } else if (isIsochronous(stats->common)) {
@@ -1518,7 +1609,7 @@ void reporter_transfer_protocol_client_tcp (struct ReporterData *data, int final
 		    stats->isochstats.cntFramesMissed = stats->isochstats.framelostcnt.current - stats->isochstats.framelostcnt.prev;
 		    stats->isochstats.cntSlips = stats->isochstats.slipcnt.current - stats->isochstats.slipcnt.prev;
 		}
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 		reporter_reset_transfer_stats_client_tcp(stats);
@@ -1539,7 +1630,7 @@ void reporter_transfer_protocol_client_tcp (struct ReporterData *data, int final
 	}
 	stats->cntBytes = stats->total.Bytes.current;
 	stats->write_mmm.current = stats->write_mmm.total;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
     } else if (isIsochronous(stats->common)) {
 	stats->isochstats.cntFrames = stats->isochstats.framecnt.current - stats->isochstats.framecnt.prev;
 	stats->isochstats.cntFramesMissed = stats->isochstats.framelostcnt.current - stats->isochstats.framelostcnt.prev;
@@ -1560,7 +1651,7 @@ void reporter_transfer_protocol_sum_client_tcp (struct TransferInfo *stats, int
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
 	if (final) {
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 		reporter_reset_transfer_stats_client_tcp(stats);
@@ -1578,7 +1669,7 @@ void reporter_transfer_protocol_sum_client_tcp (struct TransferInfo *stats, int
 	stats->sock_callstats.write.tcpstats.retry = stats->sock_callstats.write.tcpstats.retry_tot;
 #endif
 	stats->cntBytes = stats->total.Bytes.current;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
 	if ((stats->output_handler) && !(stats->isMaskOutput))
 	    (*stats->output_handler)(stats);
     }
@@ -1591,7 +1682,7 @@ void reporter_transfer_protocol_client_bb_tcp (struct ReporterData *data, int fi
 	if ((stats->cntBytes > 0) && stats->output_handler && !TimeZero(stats->ts.intervalTime)) {
 	    // print a partial interval report if enable and this a final
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 		reporter_reset_transfer_stats_client_tcp(stats);
@@ -1601,8 +1692,7 @@ void reporter_transfer_protocol_client_bb_tcp (struct ReporterData *data, int fi
 	stats->sock_callstats.write.tcpstats.retry = stats->sock_callstats.write.tcpstats.retry_tot;
 #endif
 	stats->cntBytes = stats->total.Bytes.current;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
-	stats->final=1;
+	reporter_set_timestamps_time(stats, TOTAL);
     } else {
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
     }
@@ -1619,7 +1709,7 @@ void reporter_transfer_protocol_server_bb_tcp (struct ReporterData *data, int fi
 	if ((stats->cntBytes > 0) && stats->output_handler && !TimeZero(stats->ts.intervalTime)) {
 	    // print a partial interval report if enable and this a final
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 		reporter_reset_transfer_stats_server_tcp(stats);
@@ -1630,10 +1720,8 @@ void reporter_transfer_protocol_server_bb_tcp (struct ReporterData *data, int fi
 	stats->sock_callstats.write.tcpstats.retry = stats->sock_callstats.write.tcpstats.retry_tot;
 #endif
 	stats->cntBytes = stats->total.Bytes.current;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
-	stats->final=1;
+	reporter_set_timestamps_time(stats, TOTAL);
     } else {
-	stats->final=0;
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
     }
     if ((stats->output_handler) && !(stats->isMaskOutput))
@@ -1647,13 +1735,14 @@ void reporter_transfer_protocol_sum_server_tcp (struct TransferInfo *stats, int
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
 	if (final) {
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 	    }
 	} else if ((stats->output_handler) && !(stats->isMaskOutput)) {
 	    (*stats->output_handler)(stats);
 	    stats->threadcnt = 0;
+	    stats->iInP = 0;
 	}
 	reporter_reset_transfer_stats_server_tcp(stats);
     }
@@ -1665,7 +1754,7 @@ void reporter_transfer_protocol_sum_server_tcp (struct TransferInfo *stats, int
 	    stats->sock_callstats.read.bins[ix] = stats->sock_callstats.read.totbins[ix];
 	}
 	stats->cntBytes = stats->total.Bytes.current;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
 	if ((stats->output_handler) && !(stats->isMaskOutput))
 	    (*stats->output_handler)(stats);
     }
@@ -1675,7 +1764,7 @@ void reporter_transfer_protocol_fullduplex_tcp (struct TransferInfo *stats, int
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
 	if (final) {
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 	    }
@@ -1684,9 +1773,9 @@ void reporter_transfer_protocol_fullduplex_tcp (struct TransferInfo *stats, int
     }
     if (final) {
 	stats->cntBytes = stats->total.Bytes.current;
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
     } else {
-	reporter_set_timestamps_time(&stats->ts, INTERVAL);
+	reporter_set_timestamps_time(stats, INTERVAL);
     }
     if ((stats->output_handler) && !(stats->isMaskOutput))
 	(*stats->output_handler)(stats);
@@ -1699,7 +1788,7 @@ void reporter_transfer_protocol_fullduplex_udp (struct TransferInfo *stats, int
 	stats->cntIPG = stats->total.IPG.current - stats->total.IPG.prev;
 	if (final) {
 	    if ((stats->output_handler) && !(stats->isMaskOutput)) {
-		reporter_set_timestamps_time(&stats->ts, FINALPARTIAL);
+		reporter_set_timestamps_time(stats, FINALPARTIAL);
 		if ((stats->ts.iEnd - stats->ts.iStart) > stats->ts.significant_partial)
 		    (*stats->output_handler)(stats);
 	    }
@@ -1714,9 +1803,9 @@ void reporter_transfer_protocol_fullduplex_udp (struct TransferInfo *stats, int
 	stats->cntDatagrams = stats->total.Datagrams.current ;
 	stats->cntIPG = stats->total.IPG.current;
 	stats->IPGsum = TimeDifference(stats->ts.packetTime, stats->ts.startTime);
-	reporter_set_timestamps_time(&stats->ts, TOTAL);
+	reporter_set_timestamps_time(stats, TOTAL);
     } else {
-	reporter_set_timestamps_time(&stats->ts, INTERVAL);
+	reporter_set_timestamps_time(stats, INTERVAL);
     }
     if ((stats->output_handler) && !(stats->isMaskOutput))
 	(*stats->output_handler)(stats);
@@ -1734,6 +1823,7 @@ int reporter_condprint_time_interval_report (struct ReporterData *data, struct R
     // Also signal to the caller to move to the next report (or packet ring)
     // if there was output. This will allow for more precise interval sum accounting.
     // printf("***** pt = %ld.%ld next = %ld.%ld\n", packet->packetTime.tv_sec, packet->packetTime.tv_usec, stats->ts.nextTime.tv_sec, stats->ts.nextTime.tv_usec);
+    // printf("***** nt %ld.%ld pt %ld.%ld pid=%lld empty=%d\n", stats->ts.nextTime.tv_sec, stats->ts.nextTime.tv_usec, packet->packetTime.tv_sec, packet->packetTime.tv_usec, packet->packetID, packet->emptyreport);
     if (TimeDifference(stats->ts.nextTime, packet->packetTime) < 0) {
 	assert(data->transfer_protocol_handler!=NULL);
 	advance_jobq = 1;
@@ -1743,7 +1833,7 @@ int reporter_condprint_time_interval_report (struct ReporterData *data, struct R
 #ifdef DEBUG_PPS
 	printf("*** packetID TRIGGER = %ld pt=%ld.%ld empty=%d nt=%ld.%ld\n",packet->packetID, packet->packetTime.tv_sec, packet->packetTime.tv_usec, packet->emptyreport, stats->ts.nextTime.tv_sec, stats->ts.nextTime.tv_usec);
 #endif
-	reporter_set_timestamps_time(&stats->ts, INTERVAL);
+	reporter_set_timestamps_time(stats, INTERVAL);
 	(*data->transfer_protocol_handler)(data, 0);
 	if (fullduplexstats && ((++data->FullDuplexReport->threads) == 2) && isEnhanced(stats->common)) {
 	    data->FullDuplexReport->threads = 0;
@@ -1759,7 +1849,7 @@ int reporter_condprint_time_interval_report (struct ReporterData *data, struct R
 		} else {
 		    sumstats->isMaskOutput = true;
 		}
-		reporter_set_timestamps_time(&sumstats->ts, INTERVAL);
+		reporter_set_timestamps_time(sumstats, INTERVAL);
 		assert(data->GroupSumReport->transfer_protocol_sum_handler != NULL);
 		(*data->GroupSumReport->transfer_protocol_sum_handler)(sumstats, 0);
 	    }
@@ -1811,9 +1901,9 @@ int reporter_condprint_burst_interval_report_server_tcp (struct ReporterData *da
     struct TransferInfo *stats = &data->info;
     int advance_jobq = 0;
     if (packet->transit_ready) {
-	stats->ts.prevpacketTime = packet->prevSentTime;
+	stats->ts.prevpacketTime = packet->sentTime;
 	stats->ts.packetTime = packet->packetTime;
-	reporter_set_timestamps_time(&stats->ts, INTERVALPARTIAL);
+	reporter_set_timestamps_time(stats, INTERVALPARTIAL);
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
 	if ((stats->output_handler) && !(stats->isMaskOutput))
 	    (*stats->output_handler)(stats);
@@ -1832,7 +1922,7 @@ int reporter_condprint_burst_interval_report_client_tcp (struct ReporterData *da
 //	printf("****sndpkt=%ld.%ld rxpkt=%ld.%ld\n", packet->sentTime.tv_sec, packet->sentTime.tv_usec, packet->packetTime.tv_sec,packet->packetTime.tv_usec);
 	stats->ts.prevpacketTime = packet->prevSentTime;
 	stats->ts.packetTime = packet->packetTime;
-	reporter_set_timestamps_time(&stats->ts, INTERVALPARTIAL);
+	reporter_set_timestamps_time(stats, INTERVALPARTIAL);
 	stats->cntBytes = stats->total.Bytes.current - stats->total.Bytes.prev;
 	if ((stats->output_handler) && !(stats->isMaskOutput))
 	    (*stats->output_handler)(stats);
