@@ -123,6 +123,7 @@ static void common_copy (struct ReportCommon **common, struct thread_Settings *i
 #endif
     (*common)->peer = inSettings->peer;
     (*common)->size_peer = inSettings->size_peer;
+    (*common)->jitter_binwidth = inSettings->jitter_binwidth;
     (*common)->local = inSettings->local;
     (*common)->size_local = inSettings->size_local;
     (*common)->HistBins =inSettings->mHistBins;
@@ -238,52 +239,81 @@ void SetSumHandlers (struct thread_Settings *inSettings, struct SumReport* sumre
     case kMode_Server :
 	if (isUDP(inSettings)) {
 	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_udp;
-	    if (isSumOnly(inSettings)) {
-		sumreport->info.output_handler = udp_output_sumcnt_enhanced;
-	    } else if (isFullDuplex(inSettings)) {
-		sumreport->info.output_handler = udp_output_fullduplex_sum;
+	    if (inSettings->mReportMode == kReport_CSV) {
+		if (isEnhanced(inSettings))
+		    sumreport->info.output_handler = udp_output_enhanced_csv;
+		else
+		    sumreport->info.output_handler = udp_output_basic_csv;
 	    } else {
-		sumreport->info.output_handler =  (isEnhanced(inSettings) ? udp_output_sum_read_enhanced : udp_output_sum_read);
+		if (isTripTime(inSettings)) {
+		    sumreport->info.output_handler = udp_output_sumcnt_read_triptime;
+		} else if (isSumOnly(inSettings)) {
+		    sumreport->info.output_handler = udp_output_sumcnt_enhanced;
+		} else if (isFullDuplex(inSettings)) {
+		    sumreport->info.output_handler = udp_output_fullduplex_sum;
+		} else {
+		    sumreport->info.output_handler =  (isEnhanced(inSettings) ? udp_output_sum_read_enhanced : udp_output_sum_read);
+		}
 	    }
 	} else {
 	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_server_tcp;
-	    if (isSumOnly(inSettings)) {
-		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_read_enhanced : tcp_output_sumcnt_read);
-	    } else if (isFullDuplex(inSettings)) {
-		sumreport->info.output_handler = tcp_output_sum_read;
+	    if (inSettings->mReportMode == kReport_CSV) {
+		if (isEnhanced(inSettings))
+		    sumreport->info.output_handler = tcp_output_read_enhanced_csv;
+		else
+		    sumreport->info.output_handler = tcp_output_basic_csv;
 	    } else {
-		sumreport->info.output_handler =  (isEnhanced(inSettings) ? tcp_output_sum_read_enhanced : tcp_output_sum_read);
+		if (isTripTime(inSettings)) {
+		    sumreport->info.output_handler = tcp_output_sumcnt_read_triptime;
+		} else if (isSumOnly(inSettings)) {
+		    sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_read_enhanced : tcp_output_sumcnt_read);
+		} else if (isFullDuplex(inSettings)) {
+		    sumreport->info.output_handler = tcp_output_sum_read;
+		} else {
+		    sumreport->info.output_handler =  (isEnhanced(inSettings) ? tcp_output_sum_read_enhanced : tcp_output_sum_read);
+		}
 	    }
 	}
 	break;
     case kMode_Client :
 	if (isUDP(inSettings)) {
 	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_udp;
-	    if (isSumOnly(inSettings)) {
-		sumreport->info.output_handler = ((isEnhanced(inSettings) && !isFullDuplex(inSettings)) ? \
-						  udp_output_sumcnt_write_enhanced : udp_output_sumcnt);
-	    } else if (isFullDuplex(inSettings)) {
-		sumreport->info.output_handler = udp_output_fullduplex_sum;
+	    if (inSettings->mReportMode == kReport_CSV) {
+		if (isEnhanced(inSettings))
+		    sumreport->info.output_handler = udp_output_enhanced_csv;
+		else
+		    sumreport->info.output_handler = udp_output_basic_csv;
 	    } else {
-		sumreport->info.output_handler = (isEnhanced(inSettings) ? udp_output_sum_write_enhanced : udp_output_sum_write);
+		if (isSumOnly(inSettings)) {
+		    sumreport->info.output_handler = ((isEnhanced(inSettings) && !isFullDuplex(inSettings)) ? \
+						      udp_output_sumcnt_write_enhanced : udp_output_sumcnt);
+		} else if (isFullDuplex(inSettings)) {
+		    sumreport->info.output_handler = udp_output_fullduplex_sum;
+		} else {
+		    sumreport->info.output_handler = (isEnhanced(inSettings) ? udp_output_sum_write_enhanced : udp_output_sum_write);
+		}
 	    }
 	} else {
 	    sumreport->transfer_protocol_sum_handler = reporter_transfer_protocol_sum_client_tcp;
-	    if (isSumOnly(inSettings)) {
-		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_write_enhanced : tcp_output_sumcnt_write);
-	    } else if (isFullDuplex(inSettings)) {
-		sumreport->info.output_handler = tcp_output_fullduplex_sum;
+	    if (inSettings->mReportMode == kReport_CSV) {
+		if (isEnhanced(inSettings))
+		    sumreport->info.output_handler = tcp_output_write_enhanced_csv;
+		else
+		    sumreport->info.output_handler = tcp_output_basic_csv;
 	    } else {
-		sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sum_write_enhanced : tcp_output_sum_write);
+		if (isSumOnly(inSettings)) {
+		    sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sumcnt_write_enhanced : tcp_output_sumcnt_write);
+		} else if (isFullDuplex(inSettings)) {
+		    sumreport->info.output_handler = tcp_output_fullduplex_sum;
+		} else {
+		    sumreport->info.output_handler = (isEnhanced(inSettings) ? tcp_output_sum_write_enhanced : tcp_output_sum_write);
+		}
 	    }
 	}
 	break;
     default:
 	FAIL(1, "SetSumReport", inSettings);
     }
-    // overide output handlers when csv reporting set
-    if (inSettings->mReportMode == kReport_CSV)
-	sumreport->info.output_handler = NULL;
 }
 
 struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, int fullduplex_report) {
@@ -296,9 +326,13 @@ struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, in
     Mutex_Initialize(&sumreport->reference.lock);
     sumreport->threads = 0;
     common_copy(&sumreport->info.common, inSettings);
-    sumreport->info.groupID = inID;
+    // sumreport->info.common->transferID = inID; // this is now set in the active code
     sumreport->info.threadcnt = 0;
     sumreport->info.isMaskOutput = false;
+    if (inSettings->mReportMode == kReport_CSV) {
+        format_ips_port_string(&sumreport->info, 1);
+    }
+
     // Only initialize the interval time here
     // The startTime and nextTime for summing reports will be set by
     // the reporter thread in realtime
@@ -307,6 +341,27 @@ struct SumReport* InitSumReport(struct thread_Settings *inSettings, int inID, in
 	sumreport->info.ts.intervalTime.tv_usec = (long) (inSettings->mInterval % rMillion);
 	sumreport->info.ts.significant_partial = ((double) inSettings->mInterval * PARTIALPERCENT / rMillion) ;
     }
+    // Note that for UDP the client flag settings have not been read (and set) so only use server side flags in tests
+    if (isEnhanced(inSettings) && (inSettings->mThreadMode == kMode_Server) && !fullduplex_report) {
+	if (isHistogram(inSettings)) {
+	    if (isUDP(inSettings)) {
+		char name[] = "SUMT8";
+		sumreport->info.latency_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
+								    pow(10,inSettings->mHistUnits), \
+								    inSettings->mHistci_lower, inSettings->mHistci_upper, sumreport->info.common->transferID, name);
+	    } else {
+		char name[] = "SUMF8";
+		sumreport->info.framelatency_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0, \
+									 pow(10,inSettings->mHistUnits), inSettings->mHistci_lower, \
+									 inSettings->mHistci_upper, sumreport->info.common->transferID, name);
+	    }
+	}
+	if (isJitterHistogram(inSettings) && isUDP(inSettings)) {
+	    char name[] = "SUMJ8";
+	    sumreport->info.jitter_histogram = histogram_init(JITTER_BINCNT,inSettings->jitter_binwidth,0,JITTER_UNITS, \
+							      JITTER_LCI, JITTER_UCI, sumreport->info.common->transferID, name);
+	}
+    }
     if (fullduplex_report) {
 	SetFullDuplexHandlers(inSettings, sumreport);
 	if (!isServerReverse(inSettings)) {
@@ -353,6 +408,18 @@ void FreeSumReport (struct SumReport *sumreport) {
     thread_debug("Free sum report hdr=%p", (void *)sumreport);
 #endif
     Condition_Destroy_Reference(&sumreport->reference);
+    if (sumreport->info.latency_histogram) {
+	histogram_delete(sumreport->info.latency_histogram);
+    }
+    if (sumreport->info.framelatency_histogram) {
+	histogram_delete(sumreport->info.framelatency_histogram);
+    }
+    if (sumreport->info.bbrtt_histogram) {
+	histogram_delete(sumreport->info.bbrtt_histogram);
+    }
+    if (sumreport->info.jitter_histogram) {
+	histogram_delete(sumreport->info.jitter_histogram);
+    }
     free_common_copy(sumreport->info.common);
     free(sumreport);
 }
@@ -377,6 +444,9 @@ static void Free_iReport (struct ReporterData *ireport) {
     if (ireport->info.latency_histogram) {
 	histogram_delete(ireport->info.latency_histogram);
     }
+    if (ireport->info.jitter_histogram) {
+	histogram_delete(ireport->info.jitter_histogram);
+    }
     if (ireport->info.framelatency_histogram) {
 	histogram_delete(ireport->info.framelatency_histogram);
     }
@@ -489,7 +559,6 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 
     struct ReporterData *ireport = (struct ReporterData *)(reporthdr->this_report);
     if (inSettings->mSumReport) {
-	IncrSumReportRefCounter(inSettings->mSumReport);
 	ireport->GroupSumReport = inSettings->mSumReport;
     }
     if (isFullDuplex(inSettings)) {
@@ -502,6 +571,7 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
     ireport->info.final = false;
     ireport->info.burstid_transition = false;
     ireport->info.isEnableTcpInfo = false;
+
     // Create a new packet ring which is used to communicate
     // packet stats from the traffic thread to the reporter
     // thread.  The reporter thread does all packet accounting
@@ -516,8 +586,7 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 		 (void *) ireport->packetring, ireport->packetring->bytes, (void *) ireport->packetring->awake_producer, inSettings->mSock);
 #endif
     if (inSettings->numreportstructs)
-	fprintf (stdout, "[%3d] NUM_REPORT_STRUCTS override from %d to %d\n", inSettings->mSock, NUM_REPORT_STRUCTS, inSettings->numreportstructs);
-    ireport->info.csv_peer[0] = '\0';
+	fprintf (stdout, "%sNUM_REPORT_STRUCTS override from %d to %d\n", inSettings->mTransferIDStr, NUM_REPORT_STRUCTS, inSettings->numreportstructs);
 
     // Set up the function vectors, there are three
     // 1) packet_handler: does packet accounting per the test and protocol
@@ -528,6 +597,8 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	ireport->info.ts.intervalTime.tv_usec = (long) (inSettings->mInterval % rMillion);
 	ireport->transfer_interval_handler = reporter_condprint_time_interval_report;
 	ireport->info.ts.significant_partial = (double) inSettings->mInterval * PARTIALPERCENT / rMillion ;
+    } else {
+	ireport->transfer_interval_handler = NULL;
     }
     ireport->packet_handler_pre_report = NULL;
     ireport->packet_handler_post_report = NULL;
@@ -541,17 +612,20 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 		ireport->transfer_protocol_handler = reporter_transfer_protocol_server_udp;
 	    } else {
 		ireport->transfer_protocol_handler = reporter_transfer_protocol_server_udp;
-		if (inSettings->mReportMode == kReport_CSV) {
-		    ireport->info.output_handler = udp_output_basic_csv;
-		} else if (isSumOnly(inSettings)) {
+		if (isSumOnly(inSettings)) {
 		    ireport->info.output_handler = NULL;
+		} else if ((inSettings->mReportMode == kReport_CSV) && !isSumOnly(inSettings)) {
+		    if (isEnhanced(inSettings))
+			ireport->info.output_handler = udp_output_enhanced_csv;
+		    else
+			ireport->info.output_handler = udp_output_basic_csv;
 		} else if (isTripTime(inSettings)) {
 		    if (isIsochronous(inSettings))
 			ireport->info.output_handler = udp_output_read_triptime_isoch;
 		    else
 			ireport->info.output_handler = udp_output_read_triptime;
 		} else if (isEnhanced(inSettings)) {
-		    ireport->info.output_handler = udp_output_read_triptime;
+		    ireport->info.output_handler = udp_output_read_enhanced;
 		} else if (isFullDuplex(inSettings)) {
 		    ireport->info.output_handler = udp_output_read;
 		} else {
@@ -571,14 +645,17 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 		ireport->info.output_handler = tcp_output_frame_read_triptime;
 		ireport->packet_handler_pre_report = reporter_handle_packet_server_tcp;
 		ireport->packet_handler_post_report = NULL;
-	    } else if (inSettings->mReportMode == kReport_CSV) {
-		ireport->info.output_handler = tcp_output_basic_csv;
+	    } else if (isSumOnly(inSettings)) {
+		ireport->info.output_handler = NULL;
+	    } else if ((inSettings->mReportMode == kReport_CSV) && !isSumOnly(inSettings)) {
+		if (isEnhanced(inSettings))
+		    ireport->info.output_handler = tcp_output_read_enhanced_csv;
+		else
+		    ireport->info.output_handler = tcp_output_basic_csv;
 	    } else if (isBounceBack(inSettings)) {
 		ireport->packet_handler_post_report = reporter_handle_packet_bb_server;
 		ireport->transfer_protocol_handler = reporter_transfer_protocol_server_bb_tcp;
 		ireport->info.output_handler = tcp_output_write;
-	    } else if (isSumOnly(inSettings)) {
-		ireport->info.output_handler = NULL;
 	    } else if (isTripTime(inSettings) && isIsochronous(inSettings)) {
 		ireport->info.output_handler = tcp_output_read_enhanced_isoch;
 	    } else if (isTripTime(inSettings)) {
@@ -596,10 +673,13 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	ireport->packet_handler_post_report = reporter_handle_packet_client;
 	if (isUDP(inSettings)) {
 	    ireport->transfer_protocol_handler = reporter_transfer_protocol_client_udp;
-            if (inSettings->mReportMode == kReport_CSV) {
-		ireport->info.output_handler = udp_output_basic_csv;
-	    } else if (isSumOnly(inSettings)) {
+            if (isSumOnly(inSettings)) {
 		ireport->info.output_handler = NULL;
+	    } else if ((inSettings->mReportMode == kReport_CSV) && !isSumOnly(inSettings)) {
+		if (isEnhanced(inSettings))
+		    ireport->info.output_handler = udp_output_enhanced_csv;
+		else
+		    ireport->info.output_handler = udp_output_basic_csv;
 	    } else if (isIsochronous(inSettings)) {
 		ireport->info.output_handler = udp_output_write_enhanced_isoch;
 	    } else if (isEnhanced(inSettings)) {
@@ -611,10 +691,13 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 	    }
 	} else {
 	    ireport->transfer_protocol_handler = reporter_transfer_protocol_client_tcp;
-	    if (inSettings->mReportMode == kReport_CSV) {
-		ireport->info.output_handler = tcp_output_basic_csv;
-	    } else if (isSumOnly(inSettings)) {
+	    if (isSumOnly(inSettings)) {
 		ireport->info.output_handler = NULL;
+	    } else if ((inSettings->mReportMode == kReport_CSV) && !isSumOnly(inSettings)) {
+		if (isEnhanced(inSettings))
+		    ireport->info.output_handler = tcp_output_write_enhanced_csv;
+		else
+		    ireport->info.output_handler = tcp_output_basic_csv;
 	    } else if (isBounceBack(inSettings)) {
 		ireport->packet_handler_post_report = reporter_handle_packet_bb_client;
 		ireport->transfer_protocol_handler = reporter_transfer_protocol_client_bb_tcp;
@@ -642,16 +725,23 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
 
     if (inSettings->mThreadMode == kMode_Server) {
 	ireport->info.sock_callstats.read.binsize = inSettings->mBufLen / 8;
-	if (isHistogram(inSettings) && isUDP(inSettings) && isTripTime(inSettings)) {
-	    char name[] = "T8";
-	    ireport->info.latency_histogram =  histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
-							      pow(10,inSettings->mHistUnits), \
-							      inSettings->mHistci_lower, inSettings->mHistci_upper, ireport->info.common->transferID, name);
+	if (isUDP(inSettings)) {
+	    if (isJitterHistogram(inSettings)) {
+		char name[] = "J8";
+		ireport->info.jitter_histogram = histogram_init(JITTER_BINCNT,inSettings->jitter_binwidth,0,JITTER_UNITS, \
+							      JITTER_LCI, JITTER_UCI, ireport->info.common->transferID, name);
+	    }
+	    if (isTripTime(inSettings) && isHistogram(inSettings)) {
+		char name[] = "T8";
+		ireport->info.latency_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
+								  pow(10,inSettings->mHistUnits), \
+								  inSettings->mHistci_lower, inSettings->mHistci_upper, ireport->info.common->transferID, name);
+	    }
 	}
 	if (isHistogram(inSettings) && (isIsochronous(inSettings) || (!isUDP(inSettings) && isTripTime(inSettings)))) {
 	    char name[] = "F8";
 	    // make sure frame bin size min is 100 microsecond
-	    ireport->info.framelatency_histogram =  histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0, \
+	    ireport->info.framelatency_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0, \
 								   pow(10,inSettings->mHistUnits), inSettings->mHistci_lower, \
 								   inSettings->mHistci_upper, ireport->info.common->transferID, name);
 	}
@@ -659,24 +749,26 @@ struct ReportHeader* InitIndividualReport (struct thread_Settings *inSettings) {
     if ((inSettings->mThreadMode == kMode_Client) && !isUDP(inSettings) && isHistogram(inSettings)) {
 	if (isTcpWriteTimes(inSettings)) {
 	    char name[] = "W8";
-	    ireport->info.write_histogram =  histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
+	    ireport->info.write_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
 							    pow(10,inSettings->mHistUnits), \
 							    inSettings->mHistci_lower, inSettings->mHistci_upper, ireport->info.common->transferID, name);
 	} else if (isWritePrefetch(inSettings)) {
 	    char name[] = "S8";
-	    ireport->info.latency_histogram =  histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
+	    ireport->info.latency_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,\
 							      pow(10,inSettings->mHistUnits), \
 							      inSettings->mHistci_lower, inSettings->mHistci_upper, ireport->info.common->transferID, name);
 	}
     }
     if ((inSettings->mThreadMode == kMode_Client) && isBounceBack(inSettings)) {
 	char name[] = "BB8";
-	inSettings->mHistBins = 100000; // 10 seconds wide
-	inSettings->mHistBinsize = 100; // 100 usec bins
-	inSettings->mHistUnits = 6;  // usecs 10 pow(x)
-	inSettings->mHistci_lower = 5;
-	inSettings->mHistci_upper = 95;
-	ireport->info.bbrtt_histogram =  histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,	\
+	if (!isHistogram(inSettings)) {
+	    inSettings->mHistBins = 100000; // 10 seconds wide
+	    inSettings->mHistBinsize = 100; // 100 usec bins
+	    inSettings->mHistUnits = 6;  // usecs 10 pow(x)
+	    inSettings->mHistci_lower = 5;
+	    inSettings->mHistci_upper = 95;
+	}
+	ireport->info.bbrtt_histogram = histogram_init(inSettings->mHistBins,inSettings->mHistBinsize,0,	\
 							pow(10,inSettings->mHistUnits), \
 							inSettings->mHistci_lower, inSettings->mHistci_upper, ireport->info.common->transferID, name);
     }
@@ -927,6 +1019,9 @@ void write_UDP_AckFIN (struct TransferInfo *stats, int len) {
 	    hdr->extend2.outorder_cnt2 = htonl((long) (stats->cntOutofOrder >> 32) );
 	    hdr->extend2.datagrams2    = htonl((long) (stats->cntDatagrams >> 32));
 	}
+	//	printf("****** Server final estimator %f calculated average %f\n", stats->jitter, (stats->inline_jitter.total.sum / stats->inline_jitter.total.cnt));
+	if (stats->inline_jitter.total.cnt > 0)
+	    stats->jitter = (stats->inline_jitter.total.sum / stats->inline_jitter.total.cnt); // overide the final estimator with an average
 	hdr->base.jitter1      = htonl((long) stats->jitter);
 	hdr->base.jitter2      = htonl((long) ((stats->jitter - (long)stats->jitter) * rMillion));
 
@@ -989,7 +1084,8 @@ void write_UDP_AckFIN (struct TransferInfo *stats, int len) {
 	}
 	free(ackPacket);
     }
-    if (!success)
+    if (!success && (stats->common->ReportMode != kReport_CSV)) {
 	fprintf(stderr, warn_ack_failed, stats->common->socket);
+    }
 }
 // end write_UDP_AckFIN
