@@ -73,11 +73,13 @@
 #include "payloads.h"
 #include "PerfSocket.hpp"
 #include "dscp.h"
+#include "iperf_formattime.h"
 #include <math.h>
 
 static int reversetest = 0;
 static int fullduplextest = 0;
 static int histogram = 0;
+static int jitter_histogram = 0;
 static int l2checks = 0;
 static int incrdstip = 0;
 static int incrsrcip = 0;
@@ -116,6 +118,7 @@ static int notcpbbquickack = 0;
 static int tcpquickack = 0;
 static int notcpbbquickack_cliset = 0;
 static int workingload = 0;
+static int utctimes = 0;
 static int bouncebackdelaystart = 0;
 static int tcpwritetimes = 0;
 
@@ -166,7 +169,6 @@ const struct option long_options[] =
 {"awdl",             no_argument, NULL, 'A'},
 {"bind",       required_argument, NULL, 'B'},
 {"bounceback", optional_argument, &bounceback, 1},
-{"bounceback-congest", optional_argument, &workingload, 1},
 {"bounceback-txdelay", required_argument, &bouncebackdelaystart, 1},
 {"bounceback-hold", required_argument, &bouncebackhold, 1},
 {"bounceback-no-quickack", no_argument, &notcpbbquickack, 1},
@@ -194,6 +196,7 @@ const struct option long_options[] =
 {"tcp-congestion", required_argument, NULL, 'Z'},
 {"histograms", optional_argument, &histogram, 1},
 {"hide-ips", no_argument, &hideips, 1},
+{"jitter-histograms", optional_argument, &jitter_histogram, 1},
 {"udp-histograms", optional_argument, &histogram, 1}, // keep support per 2.0.13 usage
 {"l2checks", no_argument, &l2checks, 1},
 {"incr-dstip", no_argument, &incrdstip, 1},
@@ -226,6 +229,8 @@ const struct option long_options[] =
 {"tcp-write-times", no_argument, &tcpwritetimes, 1},
 {"tap-dev", optional_argument, &tapif, 1},
 {"tun-dev", optional_argument, &tunif, 1},
+{"working-load", optional_argument, &workingload, 1},
+{"utc", no_argument, &utctimes, 1},
 {"NUM_REPORT_STRUCTS", required_argument, &numreportstructs, 1},
 #ifdef WIN32
 {"reverse", no_argument, &reversetest, 1},
@@ -366,27 +371,27 @@ void Settings_Copy (struct thread_Settings *from, struct thread_Settings **into,
     if (copyall) {
 	// Don't allocate memory for these if this is a reverse client
 	if (from->mHost != NULL) {
-	    (*into)->mHost = new char[ strlen(from->mHost) + 1];
+	    (*into)->mHost = new char[strlen(from->mHost) + 1];
 	    strcpy((*into)->mHost, from->mHost);
 	}
 	if (from->mOutputFileName != NULL) {
-	    (*into)->mOutputFileName = new char[ strlen(from->mOutputFileName) + 1];
+	    (*into)->mOutputFileName = new char[strlen(from->mOutputFileName) + 1];
 	    strcpy((*into)->mOutputFileName, from->mOutputFileName);
 	}
 	if (from->mLocalhost != NULL) {
-	    (*into)->mLocalhost = new char[ strlen(from->mLocalhost) + 1];
+	    (*into)->mLocalhost = new char[strlen(from->mLocalhost) + 1];
 	    strcpy((*into)->mLocalhost, from->mLocalhost);
 	}
 	if (from->mFileName != NULL) {
-	    (*into)->mFileName = new char[ strlen(from->mFileName) + 1];
+	    (*into)->mFileName = new char[strlen(from->mFileName) + 1];
 	    strcpy((*into)->mFileName, from->mFileName);
 	}
 	if (from->mHistogramStr != NULL) {
-	    (*into)->mHistogramStr = new char[ strlen(from->mHistogramStr) + 1];
+	    (*into)->mHistogramStr = new char[strlen(from->mHistogramStr) + 1];
 	    strcpy((*into)->mHistogramStr, from->mHistogramStr);
 	}
 	if (from->mSSMMulticastStr != NULL) {
-	    (*into)->mSSMMulticastStr = new char[ strlen(from->mSSMMulticastStr) + 1];
+	    (*into)->mSSMMulticastStr = new char[strlen(from->mSSMMulticastStr) + 1];
 	    strcpy((*into)->mSSMMulticastStr, from->mSSMMulticastStr);
 	}
 	if (from->mIfrname != NULL) {
@@ -401,6 +406,10 @@ void Settings_Copy (struct thread_Settings *from, struct thread_Settings **into,
 	    (*into)->mIsochronousStr = new char[strlen(from->mIsochronousStr) + 1];
 	    strcpy((*into)->mIsochronousStr, from->mIsochronousStr);
 	}
+	if (from->mHideHost != NULL) {
+	    (*into)->mHideHost = new char[strlen(from->mHideHost) + 1];
+	    strcpy((*into)->mHideHost, from->mHideHost);
+	}
 	if (from->mCongestion != NULL) {
 	    (*into)->mCongestion = new char[strlen(from->mCongestion) + 1];
 	    strcpy((*into)->mCongestion, from->mCongestion);
@@ -418,7 +427,7 @@ void Settings_Copy (struct thread_Settings *from, struct thread_Settings **into,
 	(*into)->mCongestion = NULL;
 	// apply the server side congestion setting to reverse clients
 	if (from->mIsochronousStr != NULL) {
-	    (*into)->mIsochronousStr = new char[ strlen(from->mIsochronousStr) + 1];
+	    (*into)->mIsochronousStr = new char[strlen(from->mIsochronousStr) + 1];
 	    strcpy((*into)->mIsochronousStr, from->mIsochronousStr);
 	}
     }
@@ -529,7 +538,7 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	setIPV4(mExtSettings);
 	break;
 
-    case '6': // v4 only
+    case '6': // v6 only
 	setIPV6(mExtSettings);
 	break;
 
@@ -555,7 +564,7 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
     setBWSet(mExtSettings);
     break;
     case 'c': // client mode w/ server host to connect to
-	mExtSettings->mHost = new char[ strlen(optarg) + 1 ];
+	mExtSettings->mHost = new char[strlen(optarg) + 1 ];
 	strcpy(mExtSettings->mHost, optarg);
 
 	if (mExtSettings->mThreadMode == kMode_Unknown) {
@@ -587,9 +596,9 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	break;
 
     case 'h': // print help and exit
-	fprintf(stderr, "%s", usage_long1);
-	fprintf(stderr, "%s", usage_long2);
-	exit(1);
+	fprintf(stdout, "%s", usage_long1);
+	fprintf(stdout, "%s", usage_long2);
+	exit(0);
 	break;
 
     case 'i': // specify interval between periodic bw reports
@@ -720,8 +729,8 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	break;
 
     case 'v': // print version and exit
-	fprintf(stderr, "%s", version);
-	exit(1);
+	fprintf(stdout, "%s", version);
+	exit(0);
 	break;
 
     case 'w': // TCP window size (socket buffer size)
@@ -783,7 +792,7 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 
     case 'B': // specify bind address
 	if (mExtSettings->mLocalhost == NULL) {
-	    mExtSettings->mLocalhost = new char[ strlen(optarg) + 1 ];
+	    mExtSettings->mLocalhost = new char[strlen(optarg) + 1 ];
 	    strcpy(mExtSettings->mLocalhost, optarg);
 	}
 	break;
@@ -1055,12 +1064,22 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	    setHistogram(mExtSettings);
 	    setEnhanced(mExtSettings);
 	    if (optarg) {
-		mExtSettings->mHistogramStr = new char[ strlen(optarg) + 1 ];
+		mExtSettings->mHistogramStr = new char[strlen(optarg) + 1 ];
 		strcpy(mExtSettings->mHistogramStr, optarg);
 	    } else {
 		mExtSettings->mHistogramStr = NULL;
 	    }
 	}
+	if (jitter_histogram) {
+	    jitter_histogram = 0;
+	    setJitterHistogram(mExtSettings);
+	    setEnhanced(mExtSettings);
+	    if (optarg) {
+		mExtSettings->jitter_binwidth = atoi(optarg);
+	    } else {
+		mExtSettings->jitter_binwidth = JITTER_BINWIDTH; // units are usecs
+	    }
+	}
 	if (reversetest) {
 	    reversetest = 0;
 	    setReverse(mExtSettings);
@@ -1097,7 +1116,7 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	    mExtSettings->mVariance = 0.0;
 	    mExtSettings->mBurstIPG = 5e-6;
 	    if (optarg) {
-		mExtSettings->mIsochronousStr = new char[ strlen(optarg) + 1 ];
+		mExtSettings->mIsochronousStr = new char[strlen(optarg) + 1 ];
 		strcpy(mExtSettings->mIsochronousStr, optarg);
 	    }
 	}
@@ -1134,6 +1153,9 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 	    setTcpQuickAck(mExtSettings);
 #endif
 	}
+	if (utctimes) {
+	    setUTC(mExtSettings);
+	}
 	if (workingload) {
 	    workingload = 0;
 #ifdef HAVE_THREAD
@@ -1152,18 +1174,18 @@ void Settings_Interpret (char option, const char *optarg, struct thread_Settings
 			    setWorkingLoadUp(mExtSettings);
 			    setWorkingLoadDown(mExtSettings);
 			} else {
-			    fprintf(stderr, "Unrecoginized value of %s for bounceback-congestion, use 'up', 'down' or 'bidir'\n", results);
+			    fprintf(stderr, "Unrecoginized value of %s for --working-load, use 'up', 'down' or 'bidir'\n", results);
 			}
 			if ((results = strtok(NULL, ",")) != NULL) {
-			    mExtSettings->mBounceBackCongestThreads = atoi(results);
+			    mExtSettings->mWorkingLoadThreads = atoi(results);
 			}
 			delete [] tmp;
 		    }
 		}
+	    }
 #else
 		fprintf(stderr, "bounceback-congest option requires a platform that supports threads\n");
 #endif
-	    }
 	}
 	if (bouncebackdelaystart) {
 	    bouncebackdelaystart = 0;
@@ -1296,7 +1318,9 @@ static void generate_permit_key (struct thread_Settings *mExtSettings) {
     int index;
     char characters[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
     for(index = timestrlength; index < (DEFAULT_PERMITKEY_LEN + timestrlength); index++) {
-	sprintf(mExtSettings->mPermitKey + index, "%c", characters[rand() % (static_cast<int>(sizeof(characters)) - 1)]);
+        int len = snprintf(mExtSettings->mPermitKey + index, sizeof(char) + 1, "%c", \
+			   characters[rand() % (static_cast<int>(sizeof(characters)) - 1)]);
+        FAIL_errno ((len < 0), "permit-key gen", mExtSettings);
     }
     mExtSettings->mPermitKey[DEFAULT_PERMITKEY_LEN + timestrlength] = '\0';
     if (strlen(mExtSettings->mPermitKey) > MAX_PERMITKEY_LEN) {
@@ -1318,10 +1342,9 @@ static void strip_v6_brackets (char *v6addr) {
 static char * isv6_bracketed_port (char *v6addr) {
     char *results = NULL;
     if (v6addr && (*v6addr ==  '[') && ((results = strtok(v6addr, "]")) != NULL)) {
+	results = strtok(NULL, ":");
 	strip_v6_brackets(v6addr);
-	if (results[0]==':') {
-	    return ++results;
-	}
+	return results;
     }
     return NULL;
 }
@@ -1459,6 +1482,10 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 	    fprintf(stderr, "WARN: option of --tcp-rx-window-clamp not supported on the client\n");
 	    unsetRxClamp(mExtSettings);
 	}
+	if (isJitterHistogram(mExtSettings)) {
+	    fprintf(stderr, "WARN: option of --jitter-histogram not supported on the client\n");
+	    unsetJitterHistogram(mExtSettings);
+	}
 	if (isIncrSrcPort(mExtSettings) && !mExtSettings->mBindPort) {
 	    fprintf(stderr, "WARN: option of --incr-srcport requires -B bind option w/port to be set\n");
 	    unsetIncrSrcPort(mExtSettings);
@@ -1480,22 +1507,19 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 	    bail = true;
 	} else if (isTxStartTime(mExtSettings) || isTxHoldback(mExtSettings)) {
 	    Timestamp now;
-	    time_t nowsecs = now.getSecs();
-	    // fill out the formats in the event they are needed per an time error
-	    char start_timebuf[80];
-	    struct tm ts = *localtime(&mExtSettings->txstart_epoch.tv_sec);
-	    strftime(start_timebuf, sizeof(start_timebuf), "(%Y-%m-%d %H:%M:%S %Z)", &ts);
-	    char now_timebuf[80];
-	    ts = *localtime(&nowsecs);
-	    strftime(now_timebuf, sizeof(now_timebuf), "%Y-%m-%d %H:%M:%S %Z", &ts);
 	    if (isTxStartTime(mExtSettings)) {
 		if (mExtSettings->txstart_epoch.tv_sec < 0) {
 		    fprintf(stderr, "ERROR: --txstart-time must be a positive value\n");
 		    unsetTxStartTime(mExtSettings);
 		    bail = true;
 		} else if ((mExtSettings->txstart_epoch.tv_sec - now.getSecs()) > MAXDIFFTXSTART) {
+		    char timestr[120];
+		    struct timeval t;
+		    t.tv_sec = now.getSecs();
+		    t.tv_usec = now.getUsecs();
+		    iperf_formattime(timestr, sizeof(timestr), t, isEnhanced(mExtSettings), isUTC(mExtSettings), YearThruSecTZ);
 		    printf(error_starttime_exceeds, mExtSettings->txstart_epoch.tv_sec, mExtSettings->txstart_epoch.tv_usec, \
-			   start_timebuf, MAXDIFFTXSTART);
+			  timestr, MAXDIFFTXSTART);
 		    bail = true;
 		}
 	    } else if (mExtSettings->txholdback_timer.tv_sec > MAXDIFFTXDELAY) {
@@ -1525,7 +1549,7 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		setPeriodicBurst(mExtSettings);
 		if (mExtSettings->mIntervalMode == kInterval_None) {
 		    mExtSettings->mIntervalMode = kInterval_Time;
-		    mExtSettings->mInterval = 1000000;
+		    mExtSettings->mInterval = 1000000; // units is us
 		}
 	    } else if (mExtSettings->mFPS <= 1e-5) {
 		unsetPeriodicBurst(mExtSettings);
@@ -1534,7 +1558,19 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		mExtSettings->mBounceBackBurst = 10;
 	    }
 	}
-
+#if HAVE_DECL_TCP_NOTSENT_LOWAT
+	if (isTripTime(mExtSettings) && !isUDP(mExtSettings)) {
+	    if (isWritePrefetch(mExtSettings)) {
+		if (mExtSettings->mWritePrefetch <= 0) {
+		    unsetWritePrefetch(mExtSettings);
+		}
+	    } else {
+		mExtSettings->mWritePrefetch = SMALL_WRITE_PREFETCH;
+		setWritePrefetch(mExtSettings);
+		setEnhanced(mExtSettings);
+	    }
+	}
+#endif
 	if (isPeriodicBurst(mExtSettings)) {
 	    if (isIsochronous(mExtSettings)) {
 		fprintf(stderr, "ERROR: options of --burst-period and --isochronous cannot be applied together\n");
@@ -1563,6 +1599,10 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		fprintf(stderr, "ERROR: option of --connect-only not supported with -u UDP\n");
 		bail = true;
 	    }
+	    if (isBounceBack(mExtSettings)) {
+		fprintf(stderr, "ERROR: option of -X or --bounceback not supported with -u UDP\n");
+		bail = true;
+	    }
 	    if (isNearCongest(mExtSettings)) {
 		fprintf(stderr, "ERROR: option of --near-congestion not supported with -u UDP\n");
 		bail = true;
@@ -1644,6 +1684,10 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 		fprintf(stderr, "ERROR: units of pps not supported with TCP\n");
 		bail = true;
 	    }
+	    if (isJitterHistogram(mExtSettings)) {
+		fprintf(stderr, "ERROR: option of --jitter-histogram not supported with TCP\n");
+		bail = true;
+	    }
 	}
 	if (!isReverse(mExtSettings) && !isFullDuplex(mExtSettings) && isHistogram(mExtSettings)){
 	    setTcpWriteTimes(mExtSettings);
@@ -1754,18 +1798,18 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
     if (bail)
 	exit(1);
 
-    // UDP histogram optional settings
+    // Histogram optional settings
     if (isHistogram(mExtSettings)) {
 	if (!mExtSettings->mHistogramStr) {
 	    if (mExtSettings->mThreadMode == kMode_Server) {
-		// set default histogram settings, milliseconds bins between 0 and 1 secs
+		// set default rx histogram settings, milliseconds bins between 0 and 1 secs
 		mExtSettings->mHistBins = 1000;
 		mExtSettings->mHistBinsize = 1;
 		mExtSettings->mHistUnits = 3;
 		mExtSettings->mHistci_lower = 5;
 		mExtSettings->mHistci_upper = 95;
 	    } else {
-		// set default histogram settings, milliseconds bins between 0 and 1 secs
+		// set default tx histogram settings, microseconds with 100 us bins
 		mExtSettings->mHistBins = 10000;
 		mExtSettings->mHistBinsize = 100;
 		mExtSettings->mHistUnits = 6;
@@ -1918,8 +1962,10 @@ void Settings_ModalOptions (struct thread_Settings *mExtSettings) {
 	}
 	mExtSettings->mIfrnametx = NULL; // default off SO_BINDTODEVICE
 	if (((results = strtok(mExtSettings->mHost, "%")) != NULL) && ((results = strtok(NULL, "%")) != NULL)) {
-	    mExtSettings->mIfrnametx = static_cast<char *>(calloc(strlen(results) + 1, sizeof(char)));
-	    strcpy(mExtSettings->mIfrnametx, results);
+	    size_t len = strlen(results) + 1;
+	    mExtSettings->mIfrnametx = static_cast<char *>(calloc(len, sizeof(char)));
+	    strncpy(mExtSettings->mIfrnametx, results, len);
+	    mExtSettings->mIfrnametx[len] = '\0';
 	}
 	if (isIPV6(mExtSettings))
 	    strip_v6_brackets(mExtSettings->mHost);
@@ -1996,7 +2042,7 @@ void Settings_GetLowerCaseArg (const char *inarg, char *outarg) {
 #define DUALTIMER_MS 300
 void Settings_GenerateListenerSettings (struct thread_Settings *client, struct thread_Settings **listener) {
     if ((client->mMode == kTest_DualTest) || (client->mMode == kTest_TradeOff)) {
-	Settings_Copy(client, listener, 0);
+	Settings_Copy(client, listener, SHALLOW_COPY);
         unsetDaemon((*listener));
         setCompat((*listener));
         if (client->mListenPort != 0) {
@@ -2107,7 +2153,7 @@ void Settings_GenerateClientSettings (struct thread_Settings *server, struct thr
     if (isFullDuplex(server) || isServerReverse(server))
 	setTransferID(server, 1);
     if (isFullDuplex(server) || v1test) {
-	Settings_Copy(server, client, 0);
+	Settings_Copy(server, client, SHALLOW_COPY);
 	reversed_thread = *client;
 	if (isFullDuplex(server) && !(flags & HEADER_VERSION1)) {
 	    setFullDuplex(reversed_thread);
@@ -2287,11 +2333,17 @@ int Settings_GenerateClientHdr (struct thread_Settings *client, void *testhdr, s
     // flags common to both TCP and UDP
     if (isReverse(client) && !isCompat(client)) {
 	upperflags |= HEADER_REVERSE;
+	if (isTxStartTime(client) && !TimeZero(startTime)) {
+	    upperflags |= HEADER_EPOCH_START;
+	}
     }
     if (isFullDuplex(client) && !isCompat(client)) {
 	upperflags |= HEADER_FULLDUPLEX;
+	if (isTxStartTime(client) && !TimeZero(startTime)) {
+	    upperflags |= HEADER_EPOCH_START;
+	}
     }
-    if (isTxStartTime(client) && !TimeZero(startTime)) {
+    if (isTxStartTime(client) && isTripTime(client) && !TimeZero(startTime)) {
 	upperflags |= HEADER_EPOCH_START;
     }
 
