@@ -61,48 +61,52 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
+
 /* -------------------------------------------------------------------
  * Create a socket address. If inHostname is not null, resolve that
  * address and fill it in. Fill in the port number. Use IPv6 ADDR_ANY
  * if that is what is desired.
  * ------------------------------------------------------------------- */
-
-void SockAddr_remoteAddr(struct thread_Settings *inSettings) {
-    SockAddr_zeroAddress(&inSettings->peer);
-    if (inSettings->mHost != NULL) {
-        SockAddr_setHostname(inSettings->mHost, &inSettings->peer, isIPV6(inSettings));
-	if (inSettings->incrdstip)
-	    SockAddr_incrAddress(&inSettings->peer, inSettings->incrdstip);
-    } else {
-#ifdef HAVE_IPV6
-        if (isIPV6(inSettings)) {
-            ((struct sockaddr*)&inSettings->peer)->sa_family = AF_INET6;
-        } else {
-            ((struct sockaddr*)&inSettings->peer)->sa_family = AF_INET;
-        }
-    }
-    if (SockAddr_isIPv6(&inSettings->peer)) {
-        inSettings->size_peer = sizeof(struct sockaddr_in6);
-    } else {
-        inSettings->size_peer = sizeof(struct sockaddr_in);
-    }
+void SockAddr_remoteAddr (struct thread_Settings *inSettings) {
+    if (SockAddr_isZeroAddress(&inSettings->peer) == 0) {
+	if (inSettings->mHost != NULL) {
+	    SockAddr_setHostname(inSettings->mHost, &inSettings->peer, isIPV6(inSettings));
+	    if (inSettings->incrdstip)
+		SockAddr_incrAddress(&inSettings->peer, inSettings->incrdstip);
+	} else {
+#if HAVE_IPV6
+	    if (isIPV6(inSettings)) {
+		((struct sockaddr*)&inSettings->peer)->sa_family = AF_INET6;
+	    } else {
+		((struct sockaddr*)&inSettings->peer)->sa_family = AF_INET;
+	    }
+	}
+	if (SockAddr_isIPv6(&inSettings->peer)) {
+	    inSettings->size_peer = sizeof(struct sockaddr_in6);
+	} else {
+	    inSettings->size_peer = sizeof(struct sockaddr_in);
+	}
 #else
+	}
         ((struct sockaddr*)&inSettings->peer)->sa_family = AF_INET;
-    }
-    inSettings->size_peer = sizeof(struct sockaddr_in);
+	inSettings->size_peer = sizeof(struct sockaddr_in);
 #endif
-    SockAddr_setPort(&inSettings->peer, inSettings->mPort);
+	SockAddr_setPort(&inSettings->peer, inSettings->mPort);
+    }
 }
 // end SocketAddr
 
-void SockAddr_localAddr(struct thread_Settings *inSettings) {
+void SockAddr_localAddr (struct thread_Settings *inSettings) {
     SockAddr_zeroAddress(&inSettings->local);
 
     if (inSettings->mLocalhost != NULL) {
         SockAddr_setHostname(inSettings->mLocalhost, &inSettings->local,
-                              isIPV6(inSettings));
+			     isIPV6(inSettings));
+	if (inSettings->incrsrcip)
+	    SockAddr_incrAddress(&inSettings->local, inSettings->incrsrcip);
+
     } else {
-#ifdef HAVE_IPV6
+#if HAVE_IPV6
         if (isIPV6(inSettings)) {
             ((struct sockaddr*)&inSettings->local)->sa_family = AF_INET6;
         } else {
@@ -147,27 +151,32 @@ void SockAddr_localAddr(struct thread_Settings *inSettings) {
 	     SockAddr_setPort(&inSettings->local, inSettings->mPort);
 	 }
      } else {
-	 /* -B was set (required to receive IP multicast) */
-	  if (inSettings->mThreadMode == kMode_Client) {
-	       /* Client thread */
-	       if (inSettings->mBindPort) {
-		   /*
-		    * User specified port so use it
-		    */
-		    SockAddr_setPort(&inSettings->local, inSettings->mBindPort);
-	       } else {
-		   /*
-		    * No user specified port, let OS assign a free one
-		    */
-		    SockAddr_setPortAny (&inSettings->local);
-	       }
-	  } else {
-	      /*
-	       * Server or Listener thread, both always use -p port
-	       * any -B port will be ignored
-	       */
-	      SockAddr_setPort(&inSettings->local, inSettings->mPort);
-	  }
+	 // -B was set
+	 if (inSettings->mThreadMode == kMode_Client) {
+	     /* Client thread */
+	     if (inSettings->mBindPort) {
+		 /*
+		  * User specified port so use it
+		  */
+#if HAVE_DECL_SO_REUSEPORT
+		 int boolean = 1;
+		 Socklen_t len = sizeof(boolean);
+		 setsockopt(inSettings->mSock, SOL_SOCKET, SO_REUSEPORT, (char*) &boolean, len);
+#endif
+		 SockAddr_setPort(&inSettings->local, (inSettings->mBindPort + inSettings->incrsrcport));
+	     } else {
+		 /*
+		  * No user specified port, let OS assign a free one
+		  */
+		 SockAddr_setPortAny (&inSettings->local);
+	     }
+	 } else {
+	     /*
+	      * Server or Listener thread, both always use -p port
+	      * any -B port will be ignored
+	      */
+	     SockAddr_setPort(&inSettings->local, inSettings->mPort);
+	 }
      }
 }
 // end SocketAddr
@@ -175,113 +184,109 @@ void SockAddr_localAddr(struct thread_Settings *inSettings) {
 /* -------------------------------------------------------------------
  * Resolve the hostname address and fill it in.
  * ------------------------------------------------------------------- */
-
-void SockAddr_setHostname(const char* inHostname, iperf_sockaddr *inSockAddr, int isIPv6) {
+void SockAddr_setHostname (const char* inHostname, iperf_sockaddr *inSockAddr, int isIPv6) {
     // ..I think this works for both ipv6 & ipv4... we'll see
-#if defined(HAVE_IPV6)
-    {
-        struct addrinfo *res, *itr;
-        int ret_ga;
-
-	if (isIPv6) {
-	    struct addrinfo hints;
-	    memset(&hints, 0, sizeof(hints));
-	    hints.ai_family = AF_INET6;
-	    ret_ga = getaddrinfo(inHostname, NULL, &hints, &res);
-	} else {
-	    ret_ga = getaddrinfo(inHostname, NULL, NULL, &res);
+    bool found = false;
+    int ret_ga;
+    struct addrinfo *res = NULL, *itr;
+    struct addrinfo hints;
+    memset(&hints, 0, sizeof(hints));
+    if (!isIPv6) {
+	hints.ai_family = AF_INET;
+	ret_ga = getaddrinfo(inHostname, NULL, &hints, &res);
+	if (ret_ga == 0) {
+	    if (res && res->ai_addr) {
+		itr = res;
+		// Now search for a IPv4 Address
+		while (itr != NULL) {
+		    if (itr->ai_family == AF_INET) {
+			memcpy(inSockAddr, (itr->ai_addr), (itr->ai_addrlen));
+			freeaddrinfo(res);
+			found = true;
+			break;
+		    } else {
+			itr = itr->ai_next;
+		    }
+		}
+	    }
 	}
-        if (ret_ga) {
-            fprintf(stderr, "error: %s\n", gai_strerror(ret_ga));
-            exit(1);
-        }
-        if (!res->ai_addr) {
-            fprintf(stderr, "getaddrinfo failed to get an address... target was '%s'\n", inHostname);
-            exit(1);
-        }
-
-        // Check address type before filling in the address
-        // ai_family = PF_xxx; ai_protocol = IPPROTO_xxx, see netdb.h
-        // ...but AF_INET6 == PF_INET6
-        itr = res;
-        if (isIPv6) {
-            // First check all results for a IPv6 Address
-            while (itr != NULL) {
-                if (itr->ai_family == AF_INET6) {
-                    memcpy(inSockAddr, (itr->ai_addr),
-                           (itr->ai_addrlen));
-                    freeaddrinfo(res);
-                    return;
-                } else {
-                    itr = itr->ai_next;
-                }
-            }
-        }
-        itr = res;
-        // Now find a IPv4 Address
-        while (itr != NULL) {
-            if (itr->ai_family == AF_INET) {
-                memcpy(inSockAddr, (itr->ai_addr),
-                       (itr->ai_addrlen));
-                freeaddrinfo(res);
-                return;
-            } else {
-                itr = itr->ai_next;
-            }
-        }
     }
-#else
-    // first try just converting dotted decimal
-    // on Windows gethostbyname doesn't understand dotted decimal
-    int rc = inet_pton(AF_INET, inHostname,
-                        (unsigned char*)&(((struct sockaddr_in*)inSockAddr)->sin_addr));
-    inSockAddr->sin_family = AF_INET;
-    if (rc == 0) {
-        struct hostent *hostP = gethostbyname(inHostname);
-        if (hostP == NULL) {
-            /* this is the same as herror() but works on more systems */
-            const char* format;
-            switch (h_errno) {
-                case HOST_NOT_FOUND:
-                    format = "%s: Unknown host\n";
-                    break;
-                case NO_ADDRESS:
-                    format = "%s: No address associated with name\n";
-                    break;
-                case NO_RECOVERY:
-                    format = "%s: Unknown server error\n";
-                    break;
-                case TRY_AGAIN:
-                    format = "%s: Host name lookup failure\n";
-                    break;
-
-                default:
-                    format = "%s: Unknown resolver error\n";
-                    break;
-            }
-            fprintf(stderr, format, inHostname);
-            exit(1);
-
-            return; // TODO throw
-        }
-
-        memcpy(&(((struct sockaddr_in*)inSockAddr)->sin_addr), *(hostP->h_addr_list),
-               (hostP->h_length));
+#if HAVE_IPV6
+    if (!found) {
+	hints.ai_family = AF_INET6;
+	ret_ga = getaddrinfo(inHostname, NULL, &hints, &res);
+	if (ret_ga == 0) {
+	    if (res && res->ai_addr) {
+		// Now search for a IPv6 Address
+		itr = res;
+		while (itr != NULL) {
+		    if (itr->ai_family == AF_INET6) {
+			memcpy(inSockAddr, (itr->ai_addr), (itr->ai_addrlen));
+			freeaddrinfo(res);
+			found = true;
+			break;
+		    } else {
+			itr = itr->ai_next;
+		    }
+		}
+	    }
+	}
+    }
+#endif // IPV6
+    // getaddrinfo didn't find an address, fallback to gethostbyname for v4
+    if (!found && !isIPv6) {
+	// first try just converting dotted decimal
+	// on Windows gethostbyname doesn't understand dotted decimal
+	struct sockaddr_in *sockaddr = (struct sockaddr_in *)inSockAddr;
+	int rc = inet_pton(AF_INET, inHostname, &sockaddr->sin_addr);
+	sockaddr->sin_family = AF_INET;
+	if (rc == 0) {
+	    struct hostent *hostP = gethostbyname(inHostname);
+	    if (hostP == NULL) {
+		/* this is the same as herror() but works on more systems */
+		const char* format;
+		switch (h_errno) {
+		case HOST_NOT_FOUND:
+		    format = "%s: Unknown host\n";
+		    break;
+		case NO_ADDRESS:
+		    format = "%s: No address associated with name\n";
+		    break;
+		case NO_RECOVERY:
+		    format = "%s: Unknown server error\n";
+		    break;
+		case TRY_AGAIN:
+		    format = "%s: Host name lookup failure\n";
+		    break;
+		default:
+		    format = "%s: Unknown resolver error\n";
+		    break;
+		}
+		fprintf(stderr, format, inHostname);
+		exit(1);
+		return; // TODO throw
+	    }
+	    found = true;
+	    memcpy(&sockaddr->sin_addr, *(hostP->h_addr_list), (hostP->h_length));
+	}
+    }
+    if (!found) {
+	fprintf(stderr, "ERROR: failed to find an ip address for host '%s'\n", inHostname);
+	exit(1);
     }
-#endif
 }
 // end setHostname
 
 /* -------------------------------------------------------------------
  * Copy the IP address into the string.
  * ------------------------------------------------------------------- */
-void SockAddr_getHostAddress(iperf_sockaddr *inSockAddr, char* outAddress,
+void SockAddr_getHostAddress (iperf_sockaddr *inSockAddr, char* outAddress,
                                 size_t len) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET) {
         inet_ntop(AF_INET, &(((struct sockaddr_in*) inSockAddr)->sin_addr),
                    outAddress, len);
     }
-#ifdef HAVE_IPV6
+#if HAVE_IPV6
     else {
         inet_ntop(AF_INET6, &(((struct sockaddr_in6*) inSockAddr)->sin6_addr),
                    outAddress, len);
@@ -293,12 +298,11 @@ void SockAddr_getHostAddress(iperf_sockaddr *inSockAddr, char* outAddress,
 /* -------------------------------------------------------------------
  * Set the address to any (generally all zeros).
  * ------------------------------------------------------------------- */
-
-void SockAddr_setAddressAny(iperf_sockaddr *inSockAddr) {
+void SockAddr_setAddressAny (iperf_sockaddr *inSockAddr) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET)
         memset(&(((struct sockaddr_in*) inSockAddr)->sin_addr), 0,
                 sizeof(struct in_addr));
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     else
         memset(&(((struct sockaddr_in6*) inSockAddr)->sin6_addr), 0,
                 sizeof(struct in6_addr));
@@ -309,11 +313,10 @@ void SockAddr_setAddressAny(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Incr the address by value
  * ------------------------------------------------------------------- */
-
-void SockAddr_incrAddress(iperf_sockaddr *inSockAddr, int value) {
+void SockAddr_incrAddress (iperf_sockaddr *inSockAddr, int value) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET)
 	((struct sockaddr_in *)inSockAddr)->sin_addr.s_addr += htonl(value);
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     else {
 	uint32_t *lower = (uint32_t *)&((struct sockaddr_in6 *)inSockAddr)->sin6_addr.s6_addr[12];
 	*lower += htonl(value);
@@ -326,11 +329,10 @@ void SockAddr_incrAddress(iperf_sockaddr *inSockAddr, int value) {
 /* -------------------------------------------------------------------
  * Set the port to the given port. Handles the byte swapping.
  * ------------------------------------------------------------------- */
-
-void SockAddr_setPort(iperf_sockaddr *inSockAddr, unsigned short inPort) {
+void SockAddr_setPort (iperf_sockaddr *inSockAddr, unsigned short inPort) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET)
         ((struct sockaddr_in*) inSockAddr)->sin_port = htons(inPort);
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     else
         ((struct sockaddr_in6*) inSockAddr)->sin6_port = htons(inPort);
 #endif
@@ -341,8 +343,7 @@ void SockAddr_setPort(iperf_sockaddr *inSockAddr, unsigned short inPort) {
 /* -------------------------------------------------------------------
  * Set the port to zero, which lets the OS pick the port.
  * ------------------------------------------------------------------- */
-
-void SockAddr_setPortAny(iperf_sockaddr *inSockAddr) {
+void SockAddr_setPortAny (iperf_sockaddr *inSockAddr) {
     SockAddr_setPort(inSockAddr, 0);
 }
 // end setPortAny
@@ -350,11 +351,10 @@ void SockAddr_setPortAny(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return the port. Handles the byte swapping.
  * ------------------------------------------------------------------- */
-
-unsigned short SockAddr_getPort(iperf_sockaddr *inSockAddr) {
+unsigned short SockAddr_getPort (iperf_sockaddr *inSockAddr) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET)
         return ntohs(((struct sockaddr_in*) inSockAddr)->sin_port);
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     else
         return ntohs(((struct sockaddr_in6*) inSockAddr)->sin6_port);
 #endif
@@ -366,8 +366,7 @@ unsigned short SockAddr_getPort(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return the IPv4 Internet Address from the sockaddr_in structure
  * ------------------------------------------------------------------- */
-
-struct in_addr* SockAddr_get_in_addr(iperf_sockaddr *inSockAddr) {
+struct in_addr* SockAddr_get_in_addr (iperf_sockaddr *inSockAddr) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET)
         return &(((struct sockaddr_in*) inSockAddr)->sin_addr);
 
@@ -378,8 +377,8 @@ struct in_addr* SockAddr_get_in_addr(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return the IPv6 Internet Address from the sockaddr_in6 structure
  * ------------------------------------------------------------------- */
-#ifdef HAVE_IPV6
-struct in6_addr* SockAddr_get_in6_addr(iperf_sockaddr *inSockAddr) {
+#if HAVE_IPV6
+struct in6_addr* SockAddr_get_in6_addr (iperf_sockaddr *inSockAddr) {
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET6)
         return &(((struct sockaddr_in6*) inSockAddr)->sin6_addr);
 
@@ -392,10 +391,8 @@ struct in6_addr* SockAddr_get_in6_addr(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return the size of the appropriate address structure.
  * ------------------------------------------------------------------- */
-
-Socklen_t SockAddr_get_sizeof_sockaddr(iperf_sockaddr *inSockAddr) {
-
-#if defined(HAVE_IPV6)
+Socklen_t SockAddr_get_sizeof_sockaddr (iperf_sockaddr *inSockAddr) {
+#if HAVE_IPV6
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET6) {
         return(sizeof(struct sockaddr_in6));
     }
@@ -408,10 +405,8 @@ Socklen_t SockAddr_get_sizeof_sockaddr(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return if IPv6 socket
  * ------------------------------------------------------------------- */
-
-int SockAddr_isIPv6(iperf_sockaddr *inSockAddr) {
-
-#if defined(HAVE_IPV6)
+int SockAddr_isIPv6 (iperf_sockaddr *inSockAddr) {
+#if HAVE_IPV6
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET6) {
         return 1;
     }
@@ -423,30 +418,33 @@ int SockAddr_isIPv6(iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Return true if the address is multicast ip address.
  * ------------------------------------------------------------------- */
-
-int SockAddr_isMulticast(iperf_sockaddr *inSockAddr) {
-
-#if defined(HAVE_IPV6)
+int SockAddr_isMulticast (iperf_sockaddr *inSockAddr) {
+#if HAVE_IPV6
     if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET6) {
         return(IN6_IS_ADDR_MULTICAST(&(((struct sockaddr_in6*) inSockAddr)->sin6_addr)));
     } else
 #endif
-    {
-        // 224.0.0.0 to 239.255.255.255 (e0.00.00.00 to ef.ff.ff.ff)
-        const unsigned long kMulticast_Mask = 0xe0000000L;
+	{
 
-        return(kMulticast_Mask ==
-               (ntohl(((struct sockaddr_in*) inSockAddr)->sin_addr.s_addr) & kMulticast_Mask));
-    }
+#ifdef IN_MULTICAST
+	    // 224.0.0.0 to 239.255.255.255 (e0.00.00.00 to ef.ff.ff.ff)
+	    // convert from network to host byte order
+	    uint32_t maddr = ntohl((uint32_t)(((struct sockaddr_in*) inSockAddr)->sin_addr.s_addr));
+	    return (IN_MULTICAST(maddr));
+#else
+	    const unsigned long kClassD_Mask = 0xf0000000L;
+	    const unsigned long kMulticast = 0xe0000000L;
+	    return((((ntohl(((struct sockaddr_in*) inSockAddr)->sin_addr.s_addr)) & kClassD_Mask) == kMulticast));
+#endif
+	}
 }
 // end isMulticast
 
 /* -------------------------------------------------------------------
  * Return true if the address is multicast ip address.
  * ------------------------------------------------------------------- */
-
 int SockAddr_isLinklocal (iperf_sockaddr *inSockAddr) {
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
   if (((struct sockaddr*)inSockAddr)->sa_family == AF_INET6) {
       return(IN6_IS_ADDR_LINKLOCAL(&(((struct sockaddr_in6*) inSockAddr)->sin6_addr)));
     } else
@@ -459,29 +457,26 @@ int SockAddr_isLinklocal (iperf_sockaddr *inSockAddr) {
 /* -------------------------------------------------------------------
  * Zero out the address structure.
  * ------------------------------------------------------------------- */
-
-void SockAddr_zeroAddress(iperf_sockaddr *inSockAddr) {
+void SockAddr_zeroAddress (iperf_sockaddr *inSockAddr) {
     memset(inSockAddr, 0, sizeof(iperf_sockaddr));
 }
 
-int SockAddr_isZeroAddress(iperf_sockaddr *inSockAddr) {
+int SockAddr_isZeroAddress (iperf_sockaddr *inSockAddr) {
     iperf_sockaddr zeroSockAddr;
     memset(&zeroSockAddr, 0, sizeof(iperf_sockaddr));
     return(memcmp((void *)inSockAddr, (void *)&zeroSockAddr, sizeof(iperf_sockaddr)));
 }
 
-// zeroAddress
-
 /* -------------------------------------------------------------------
  * Compare two sockaddrs and return true if they are equal
  * ------------------------------------------------------------------- */
-int SockAddr_are_Equal(iperf_sockaddr *first, iperf_sockaddr *second) {
+int SockAddr_are_Equal (iperf_sockaddr *first, iperf_sockaddr *second) {
     if (((struct sockaddr*)first)->sa_family == AF_INET && ((struct sockaddr*)second)->sa_family == AF_INET) {
         // compare IPv4 adresses
         return(((long) ((struct sockaddr_in*)first)->sin_addr.s_addr == (long) ((struct sockaddr_in*)second)->sin_addr.s_addr)
                 && (((struct sockaddr_in*)first)->sin_port == ((struct sockaddr_in*)second)->sin_port));
     }
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     if (((struct sockaddr*)first)->sa_family == AF_INET6 && ((struct sockaddr*)second)->sa_family == AF_INET6) {
         // compare IPv6 addresses
         return(!memcmp(((struct sockaddr_in6*)first)->sin6_addr.s6_addr, ((struct sockaddr_in6*)second)->sin6_addr.s6_addr, sizeof(struct in6_addr))
@@ -489,19 +484,18 @@ int SockAddr_are_Equal(iperf_sockaddr *first, iperf_sockaddr *second) {
     }
 #endif
     return 0;
-
 }
 
 /* -------------------------------------------------------------------
  * Compare two sockaddrs and return true if the hosts are equal
  * ------------------------------------------------------------------- */
-int SockAddr_Hostare_Equal(iperf_sockaddr* first, iperf_sockaddr *second) {
+int SockAddr_Hostare_Equal (iperf_sockaddr* first, iperf_sockaddr *second) {
     if (((struct sockaddr*)first)->sa_family == AF_INET && ((struct sockaddr*)second)->sa_family == AF_INET) {
         // compare IPv4 adresses
         return((long) ((struct sockaddr_in*)first)->sin_addr.s_addr ==
                 (long) ((struct sockaddr_in*)second)->sin_addr.s_addr);
     }
-#if defined(HAVE_IPV6)
+#if HAVE_IPV6
     if (((struct sockaddr*)first)->sa_family == AF_INET6 && ((struct sockaddr*)second)->sa_family == AF_INET6) {
         // compare IPv6 addresses
         return(!memcmp(((struct sockaddr_in6*)first)->sin6_addr.s6_addr,
@@ -509,8 +503,8 @@ int SockAddr_Hostare_Equal(iperf_sockaddr* first, iperf_sockaddr *second) {
     }
 #endif
     return 0;
-
 }
+
 /* -------------------------------------------------------------------
  * Find the interface name of a connected socket (when not already set)
  * Can be forced with -B <ip>%<name> (server), -c <ip>%<name> (client)
@@ -521,7 +515,7 @@ int SockAddr_Hostare_Equal(iperf_sockaddr* first, iperf_sockaddr *second) {
  * Store (and cache) the results in the thread settings structure
  * Return 0 if set, -1 if not
  * ------------------------------------------------------------------- */
-int SockAddr_Ifrname(struct thread_Settings *inSettings) {
+int SockAddr_Ifrname (struct thread_Settings *inSettings) {
 #ifdef HAVE_IFADDRS_H
     if (inSettings->mIfrname == NULL) {
 	struct sockaddr_storage myaddr;
@@ -610,17 +604,34 @@ int SockAddr_Drop_All_BPF (int sock) {
     return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
 }
 
-int SockAddr_Accept_BPF (int sock, uint16_t port) {
+//
+// [root@ryzen3950 iperf2-code]# tcpdump ip and udp dst port 5001 -e -dd -i any
+// tcpdump: data link type LINUX_SLL2
+// { 0x28, 0, 0, 0x00000000 },
+// { 0x15, 0, 8, 0x00000800 },
+// { 0x30, 0, 0, 0x0000001d },
+// { 0x15, 0, 6, 0x00000011 },
+// { 0x28, 0, 0, 0x0000001a },
+// { 0x45, 4, 0, 0x00001fff },
+// { 0xb1, 0, 0, 0x00000014 },
+// { 0x48, 0, 0, 0x00000016 },
+// { 0x15, 0, 1, 0x00001389 },
+// { 0x6, 0, 0, 0x00040000 },
+// { 0x6, 0, 0, 0x00000000 },
+
+//	{ 0x28, 0, 0, 0x00000014 },
+//	{ 0x45, 4, 0, 0x00001fff },
+//	{ 0xb1, 0, 0, 0x0000001e },
+//	{ 0x48, 0, 0, 0x00000010 },
+//	{ 0x15, 0, 1, 0x00001389 },
+
+
+int SockAddr_v4_Accept_BPF (int sock, uint16_t port) {
     // tcpdump udp dst port 5001 -dd to get c code filter
     // UDP port is the 5 and 13 bytecodes (5001 = 0x1389)
     // see linux/filter.h
     struct sock_filter udp_filter[] = {
 	{ 0x28, 0, 0, 0x0000000c },
-	{ 0x15, 0, 4, 0x000086dd },
-	{ 0x30, 0, 0, 0x00000014 },
-	{ 0x15, 0, 11, 0x00000011 },
-	{ 0x28, 0, 0, 0x00000038 },
-	{ 0x15, 8, 9, 0x00001389 },
 	{ 0x15, 0, 8, 0x00000800 },
 	{ 0x30, 0, 0, 0x00000017 },
 	{ 0x15, 0, 6, 0x00000011 },
@@ -632,15 +643,58 @@ int SockAddr_Accept_BPF (int sock, uint16_t port) {
 	{ 0x6, 0, 0, 0x00040000 },
 	{ 0x6, 0, 0, 0x00000000 },
     };
-    udp_filter[5].k = port;
-    udp_filter[13].k = port;
+    udp_filter[8].k = port;
     struct sock_fprog bpf = {
 	.len = (sizeof(udp_filter) / sizeof(struct sock_filter)),
 	.filter = udp_filter,
     };
     return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
 }
+
+//[root@ryzen3950 iperf2-code]# tcpdump  udp dst port 5001 and dst host 1.1.1.1 -dd
+//Warning: assuming Ethernet
+//{ 0x28, 0, 0, 0x0000000c },
+//{ 0x15, 11, 0, 0x000086dd },
+//{ 0x15, 0, 10, 0x00000800 },
+//{ 0x30, 0, 0, 0x00000017 },
+//{ 0x15, 0, 8, 0x00000011 },
+//{ 0x28, 0, 0, 0x00000014 },
+//{ 0x45, 6, 0, 0x00001fff },
+//{ 0xb1, 0, 0, 0x0000000e },
+//{ 0x48, 0, 0, 0x00000010 },
+//{ 0x15, 0, 3, 0x00001389 },
+//{ 0x20, 0, 0, 0x0000001e },
+//{ 0x15, 0, 1, 0x01010101 },
+//{ 0x6, 0, 0, 0x00040000 },
+//{ 0x6, 0, 0, 0x00000000 },
 //
+// BPF for TAP interaces with explicit v4 IP and UDP dst port
+int SockAddr_Accept_V4_TAP_BPF (int sock, uint32_t dstip, uint16_t port) {
+    struct sock_filter udp_filter[] = {
+	{ 0x28, 0, 0, 0x0000000c },
+	{ 0x15, 11, 0, 0x000086dd },
+	{ 0x15, 0, 10, 0x00000800 },
+	{ 0x30, 0, 0, 0x00000017 },
+	{ 0x15, 0, 8, 0x00000011 },
+	{ 0x28, 0, 0, 0x00000014 },
+	{ 0x45, 6, 0, 0x00001fff },
+	{ 0xb1, 0, 0, 0x0000000e },
+	{ 0x48, 0, 0, 0x00000010 },
+	{ 0x15, 0, 3, 0x00001389 },
+	{ 0x20, 0, 0, 0x0000001e },
+	{ 0x15, 0, 1, 0x00000000 },
+	{ 0x6, 0, 0, 0x00040000 },
+	{ 0x6, 0, 0, 0x00000000 },
+    };
+    udp_filter[12].k = htonl(dstip);
+    udp_filter[9].k = htons(port);
+    struct sock_fprog bpf = {
+	.len = (sizeof(udp_filter) / sizeof(struct sock_filter)),
+	.filter = udp_filter,
+    };
+    return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
+}
+
 // Simulate the UDP connect for the AF_PACKET (or PF_PACKET)
 //
 int SockAddr_v4_Connect_BPF (int sock, uint32_t dstip, uint32_t srcip, uint16_t dstport, uint16_t srcport) {
@@ -698,6 +752,57 @@ int SockAddr_v4_Connect_BPF (int sock, uint32_t dstip, uint32_t srcip, uint16_t
     };
     return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
 }
+int SockAddr_v4_Connect_TAP_BPF (int sock, uint32_t dstip, uint32_t srcip, uint16_t dstport, uint16_t srcport) {
+    // [root@ryzen3950 iperf2-code]# tcpdump ip and src 127.0.0.1 and dst 127.0.0.2 and udp src port 5001 and dst port 5002  -d
+    // Warning: assuming Ethernet
+    // (000) ldh      [12]
+    // (001) jeq      #0x800           jt 2	jf 16
+    // (002) ld       [26]
+    // (003) jeq      #0x7f000001      jt 4	jf 16
+    // (004) ld       [30]
+    // (005) jeq      #0x7f000002      jt 6	jf 16
+    // (006) ldb      [23]
+    // (007) jeq      #0x11            jt 8	jf 16
+    // (008) ldh      [20]
+    // (009) jset     #0x1fff          jt 16	jf 10
+    // (010) ldxb     4*([14]&0xf)
+    // (011) ldh      [x + 14]
+    // (012) jeq      #0x1389          jt 13	jf 16
+    // (013) ldh      [x + 16]
+    // (014) jeq      #0x138a          jt 15	jf 16
+    // (015) ret      #262144
+    // (016) ret      #0
+    // [root@ryzen3950 iperf2-code]# tcpdump ip and src 127.0.0.1 and dst 127.0.0.2 and udp src port 5001 and dst port 5002  -dd
+    // Warning: assuming Ethernet
+    struct sock_filter udp_filter[] = {
+	{ 0x28, 0, 0, 0x0000000c },
+	{ 0x15, 0, 14, 0x00000800 },
+	{ 0x20, 0, 0, 0x0000001a },
+	{ 0x15, 0, 12, 0x7f000001 },
+	{ 0x20, 0, 0, 0x0000001e },
+	{ 0x15, 0, 10, 0x7f000002 },
+	{ 0x30, 0, 0, 0x00000017 },
+	{ 0x15, 0, 8, 0x00000011 },
+	{ 0x28, 0, 0, 0x00000014 },
+	{ 0x45, 6, 0, 0x00001fff },
+	{ 0xb1, 0, 0, 0x0000000e },
+	{ 0x48, 0, 0, 0x0000000e },
+	{ 0x15, 0, 3, 0x00001389 },
+	{ 0x48, 0, 0, 0x00000010 },
+	{ 0x15, 0, 1, 0x0000138a },
+	{ 0x6, 0, 0, 0x00040000 },
+	{ 0x6, 0, 0, 0x00000000 },
+    };
+    udp_filter[3].k = htonl(srcip);
+    udp_filter[5].k = htonl(dstip);
+    udp_filter[12].k = htons(srcport);
+    udp_filter[14].k = htons(dstport);
+    struct sock_fprog bpf = {
+	.len = (sizeof(udp_filter) / sizeof(struct sock_filter)),
+	.filter = udp_filter,
+    };
+    return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
+}
 
 int SockAddr_v4_Connect_BPF_Drop (int sock, uint32_t dstip, uint32_t srcip, uint16_t dstport, uint16_t srcport) {
     // Use full quintuple, proto, src ip, dst ip, src port, dst port
@@ -734,7 +839,7 @@ int SockAddr_v4_Connect_BPF_Drop (int sock, uint32_t dstip, uint32_t srcip, uint
     };
     return(setsockopt(sock, SOL_SOCKET, SO_ATTACH_FILTER, &bpf, sizeof(bpf)));
 }
-#  ifdef HAVE_IPV6
+#if HAVE_IPV6
 //
 // v6 Connected BPF, use 32 bit values
 //
