@@ -114,14 +114,17 @@ static void active_table_update (iperf_sockaddr *host, struct thread_Settings *a
 	active_table.count++;
 	active_table.groupid++;
 	active_table.root = this_entry;
+	this_entry->sum_report = InitSumReport(agent, active_table.total_count, 0);
+	IncrSumReportRefCounter(this_entry->sum_report);
+	agent->mSumReport = this_entry->sum_report;
+	this_entry->sum_report->info.common->transferID = -active_table.groupid; // sum ids are negative
 #if HAVE_THREAD_DEBUG
 	active_table_show_entry("new entry", this_entry, ((SockAddr_are_Equal(&this_entry->host, host) && SockAddr_Hostare_Equal(&this_entry->host, host))));
 #endif
-	this_entry->sum_report = InitSumReport(agent, active_table.total_count, 0);
-	agent->mSumReport = this_entry->sum_report;
     } else {
 	this_entry->thread_count++;
 	agent->mSumReport = this_entry->sum_report;
+	IncrSumReportRefCounter(this_entry->sum_report);
 #if HAVE_THREAD_DEBUG
 	active_table_show_entry("incr entry", this_entry, 1);
 #endif
@@ -180,12 +183,15 @@ void Iperf_remove_host (struct thread_Settings *agent) {
 	if (--(*tmp)->thread_count == 0) {
 	    Iperf_ListEntry *remove = (*tmp);
 	    active_table.count--;
+	    agent->mSumReport = NULL;
 #if HAVE_THREAD_DEBUG
 	    active_table_show_entry("delete", remove, 1);
 #endif
 	    *tmp = remove->next;
+	    FreeSumReport(remove->sum_report);
 	    delete remove;
 	} else {
+	    DecrSumReportRefCounter((*tmp)->sum_report);
 #if HAVE_THREAD_DEBUG
 	    active_table_show_entry("decr", (*tmp), 1);
 #endif
