@@ -118,8 +118,8 @@ int FrameCounter::mySetWaitableTimer (long delay_time) {
 }
 #endif
 
-#if defined(HAVE_CLOCK_NANOSLEEP)
-unsigned int FrameCounter::wait_tick () {
+#if HAVE_CLOCK_NANOSLEEP
+unsigned int FrameCounter::wait_tick (long *sched_err) {
     Timestamp now;
     int rc = true;
     if (!slot_counter) {
@@ -130,6 +130,7 @@ unsigned int FrameCounter::wait_tick () {
 	while (!now.before(nextslotTime)) {
 	    now.setnow();
 	    nextslotTime.add(period);
+//	    printf("***** next slot %ld.%ld\n",nextslotTime.getSecs(), nextslotTime.getUsecs());
 	    slot_counter++;
 	}
 	if (lastcounter && ((slot_counter - lastcounter) > 1)) {
@@ -143,9 +144,33 @@ unsigned int FrameCounter::wait_tick () {
     rc = clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &txtime_ts, NULL);
   #else
     long duration = nextslotTime.subUsec(now);
-    rc = mySetWaitableTimer(10 * duration); // convert us to 100ns
+    rc = mySetWaitableTimer(10 * duration); // convert us to 100 ns
     //int rc = clock_nanosleep(0, TIMER_ABSTIME, &txtime_ts, NULL);
   #endif
+    if (sched_err) {
+        // delay_loop(2020);
+	Timestamp actual;
+	*sched_err = actual.subUsec(nextslotTime);
+//	printf("**** sched err %ld\n", *sched_err);
+	if (*sched_err < 0) {
+	    *sched_err = -(*sched_err); // err is an absolute value
+            // Per windows docs, this timer can go off early per:
+            // APIs that deal with timers use various different hardware clocks. These clocks may have resolutions
+	    // significantly different from what you expect: some may be measured in milliseconds (for those that
+	    // use an RTC-based timer chip), to those measured in nanoseconds (for those that use ACPI or TSC counters).
+	    // You can change the resolution of your API with a call to the timeBeginPeriod and timeEndPeriod functions.
+	    // How precise you can change the resolution depends on which hardware clock the particular API uses.
+	    // For more information, check your hardware documentation.
+            //
+	    // I noticed the timer going off up to 8 ms early on a Windows 11 cross compile - yikes.
+	    // Do a WAR hack here to add delay if & when that occurs
+#ifdef WIN32
+	    if (*sched_err > 1000) {
+		delay_loop(*sched_err);
+	    }
+#endif
+	}
+    }
     WARN_errno((rc!=0), "wait_tick failed");
   #ifdef HAVE_THREAD_DEBUG
     // thread_debug("Client tick occurred per %ld.%ld", txtime_ts.tv_sec, txtime_ts.tv_nsec / 1000);
@@ -154,7 +179,7 @@ unsigned int FrameCounter::wait_tick () {
     return(slot_counter);
 }
 #else
-unsigned int FrameCounter::wait_tick (void) {
+unsigned int FrameCounter::wait_tick (long *sched_err) {
     long remaining;
     unsigned int framecounter;
 
@@ -174,7 +199,16 @@ unsigned int FrameCounter::wait_tick (void) {
 	    tv0.tv_sec++;
 	    tv0.tv_nsec -= BILLION;
 	}
+	Timestamp slotstart = startTime;
+	slot_counter = get();
+	// period unit is in microseconds, convert to seconds
+	slotstart.add(slot_counter * period * 1e-6);
 	int rc = nanosleep(&tv0, &tv1);
+	if (sched_err) {
+	    Timestamp actual;
+	    *sched_err = actual.subUsec(slotstart);
+//	    printf("**** slot %ld.%ld actual %ld.%ld %ld\n", slotstart.getSecs(), slotstart.getUsecs(), actual.getSecs(), actual.getUsecs(), *sched_err);
+	}
 	WARN_errno((rc != 0), "nanosleep wait_tick");	    ;
 //	printf("****** rc = %d, remain %ld.%ld\n", rc, tv1.tv_sec, tv1.tv_nsec);
 	framecounter ++;
